@@ -17,14 +17,26 @@
 
 #include "http/action/pad_rowset_action.h"
 
+#include <gen_cpp/olap_file.pb.h>
+#include <glog/logging.h>
+#include <stdint.h>
+
+#include <cstdlib>
 #include <memory>
 #include <mutex>
+#include <ostream>
+#include <string>
+#include <vector>
 
 #include "http/http_channel.h"
+#include "http/http_request.h"
+#include "http/http_status.h"
 #include "olap/olap_common.h"
-#include "olap/rowset/beta_rowset_writer.h"
 #include "olap/rowset/rowset.h"
+#include "olap/rowset/rowset_writer.h"
+#include "olap/rowset/rowset_writer_context.h"
 #include "olap/storage_engine.h"
+#include "olap/tablet_manager.h"
 #include "util/time.h"
 #include "util/trace.h"
 
@@ -88,8 +100,13 @@ Status PadRowsetAction::_pad_rowset(TabletSharedPtr tablet, const Version& versi
     }
 
     std::unique_ptr<RowsetWriter> writer;
-    RETURN_IF_ERROR(tablet->create_rowset_writer(version, VISIBLE, NONOVERLAPPING,
-                                                 tablet->tablet_schema(), -1, -1, &writer));
+    RowsetWriterContext ctx;
+    ctx.version = version;
+    ctx.rowset_state = VISIBLE;
+    ctx.segments_overlap = NONOVERLAPPING;
+    ctx.tablet_schema = tablet->tablet_schema();
+    ctx.newest_write_timestamp = UnixSeconds();
+    RETURN_IF_ERROR(tablet->create_rowset_writer(ctx, &writer));
     auto rowset = writer->build();
     rowset->make_visible(version);
 
