@@ -17,9 +17,15 @@
 
 #pragma once
 
-#include "gen_cpp/PlanNodes_types.h"
-#include "io/file_reader.h"
-#include "io/fs/hdfs.h"
+#include <gen_cpp/PlanNodes_types.h>
+
+#include <map>
+#include <string>
+
+#include "common/status.h"
+#include "io/fs/hdfs.h" // IWYU pragma: keep
+
+struct hdfsBuilder;
 
 namespace doris {
 
@@ -35,9 +41,9 @@ class HDFSCommonBuilder {
                                     HDFSCommonBuilder* builder);
 
 public:
-    HDFSCommonBuilder() = default;
+    HDFSCommonBuilder() {}
     ~HDFSCommonBuilder() {
-#if defined(USE_LIBHDFS3) || defined(BE_TEST)
+#ifdef USE_LIBHDFS3
         // for hadoop hdfs, the hdfs_builder will be freed in hdfsConnect
         if (hdfs_builder != nullptr) {
             hdfsFreeBuilder(hdfs_builder);
@@ -48,8 +54,8 @@ public:
     // Must call this to init hdfs_builder first.
     Status init_hdfs_builder();
 
-    hdfsBuilder* get() { return hdfs_builder; };
-    bool is_need_kinit() { return need_kinit; };
+    hdfsBuilder* get() { return hdfs_builder; }
+    bool is_need_kinit() const { return need_kinit; }
     Status run_kinit();
 
 private:
