@@ -17,28 +17,32 @@
 
 #pragma once
 
+#include <gen_cpp/Types_types.h>
+#include <stddef.h>
+
+#include <atomic>
 #include <condition_variable>
 #include <cstdint>
+#include <memory>
 #include <mutex>
 #include <set>
 #include <shared_mutex>
 #include <string>
+#include <vector>
 
 #include "common/status.h"
-#include "env/env.h"
-#include "gen_cpp/Types_types.h"
-#include "gen_cpp/olap_file.pb.h"
 #include "io/fs/file_system.h"
+#include "io/fs/fs_utils.h"
 #include "olap/olap_common.h"
-#include "olap/rowset/rowset_id_generator.h"
 #include "util/metrics.h"
 
 namespace doris {
 
 class Tablet;
 class TabletManager;
-class TabletMeta;
 class TxnManager;
+class OlapMeta;
+class RowsetIdGenerator;
 
 // A DataDir used to manage data in same path.
 // Now, After DataDir was created, it will never be deleted for easy implementation.
@@ -58,7 +62,6 @@ public:
     const io::FileSystemSPtr& fs() const { return _fs; }
 
     bool is_used() const { return _is_used; }
-    void set_is_used(bool is_used) { _is_used = is_used; }
     int32_t cluster_id() const { return _cluster_id; }
     bool cluster_id_incomplete() const { return _cluster_id_incomplete; }
 
@@ -85,8 +88,6 @@ public:
 
     bool is_ssd_disk() const { return _storage_medium == TStorageMedium::SSD; }
 
-    bool is_remote() const { return FilePathDesc::is_remote(_storage_medium); }
-
     TStorageMedium::type storage_medium() const { return _storage_medium; }
 
     void register_tablet(Tablet* tablet);
@@ -110,7 +111,7 @@ public:
 
     // this function scans the paths in data dir to collect the paths to check
     // this is a producer function. After scan, it will notify the perform_path_gc function to gc
-    void perform_path_scan();
+    Status perform_path_scan();
 
     void perform_path_gc_by_rowsetid();
 
@@ -145,12 +146,12 @@ public:
 
 private:
     Status _init_cluster_id();
-    Status _init_capacity();
+    Status _init_capacity_and_create_shards();
     Status _init_meta();
 
     Status _check_disk();
     Status _read_and_write_test_file();
-    Status read_cluster_id(Env* env, const std::string& cluster_id_path, int32_t* cluster_id);
+    Status read_cluster_id(const std::string& cluster_id_path, int32_t* cluster_id);
     Status _write_cluster_id_to_path(const std::string& path, int32_t cluster_id);
     // Check whether has old format (hdr_ start) in olap. When doris updating to current version,
     // it may lead to data missing. When conf::storage_strict_check_incompatible_old_format is true,
@@ -170,14 +171,10 @@ private:
     size_t _path_hash;
 
     io::FileSystemSPtr _fs;
-    // user specified capacity
-    int64_t _capacity_bytes;
     // the actual available capacity of the disk of this data dir
-    // NOTICE that _available_bytes may be larger than _capacity_bytes, if capacity is set
-    // by user, not the disk's actual capacity
-    int64_t _available_bytes;
+    size_t _available_bytes;
     // the actual capacity of the disk of this data dir
-    int64_t _disk_capacity_bytes;
+    size_t _disk_capacity_bytes;
     TStorageMedium::type _storage_medium;
     bool _is_used;
 
