@@ -17,9 +17,10 @@
 
 #pragma once
 
-#include "gen_cpp/olap_file.pb.h"
-#include "olap/data_dir.h"
-#include "olap/storage_engine.h"
+#include <gen_cpp/olap_file.pb.h>
+
+#include "io/fs/file_system.h"
+#include "olap/olap_define.h"
 #include "olap/tablet.h"
 #include "olap/tablet_schema.h"
 
@@ -27,15 +28,18 @@ namespace doris {
 
 class RowsetWriterContextBuilder;
 using RowsetWriterContextBuilderSharedPtr = std::shared_ptr<RowsetWriterContextBuilder>;
+class DataDir;
+class Tablet;
+namespace vectorized::schema_util {
+class LocalSchemaChangeRecorder;
+}
 
 struct RowsetWriterContext {
     RowsetWriterContext()
             : tablet_id(0),
               tablet_schema_hash(0),
               partition_id(0),
-              rowset_type(ALPHA_ROWSET),
-              fs(nullptr),
-              tablet_schema(nullptr),
+              rowset_type(BETA_ROWSET),
               rowset_state(PREPARED),
               version(Version(0, 0)),
               txn_id(0),
@@ -45,33 +49,13 @@ struct RowsetWriterContext {
         load_id.set_lo(0);
     }
 
-    static RowsetWriterContext create(const Version& version, TabletSharedPtr new_tablet,
-                                      RowsetTypePB new_rowset_type,
-                                      SegmentsOverlapPB segments_overlap) {
-        RowsetWriterContext context;
-        context.rowset_id = StorageEngine::instance()->next_rowset_id();
-        context.tablet_uid = new_tablet->tablet_uid();
-        context.tablet_id = new_tablet->tablet_id();
-        context.partition_id = new_tablet->partition_id();
-        context.tablet_schema_hash = new_tablet->schema_hash();
-        context.rowset_type = new_rowset_type;
-        context.rowset_dir = new_tablet->tablet_path();
-        context.tablet_schema = new_tablet->tablet_schema();
-        context.data_dir = new_tablet->data_dir();
-        context.rowset_state = VISIBLE;
-        context.version = version;
-        context.segments_overlap = segments_overlap;
-
-        return context;
-    }
-
     RowsetId rowset_id;
     int64_t tablet_id;
     int64_t tablet_schema_hash;
     int64_t partition_id;
     RowsetTypePB rowset_type;
-    io::FileSystemSPtr fs = nullptr;
-    std::string rowset_dir = "";
+    io::FileSystemSPtr fs;
+    std::string rowset_dir;
     TabletSchemaSPtr tablet_schema;
     // PREPARED/COMMITTED for pending rowset
     // VISIBLE for non-pending rowset
@@ -96,9 +80,16 @@ struct RowsetWriterContext {
     // (because it hard to refactor, and RowsetConvertor will be deprecated in future)
     DataDir* data_dir = nullptr;
 
-    int64_t oldest_write_timestamp;
     int64_t newest_write_timestamp;
     bool enable_unique_key_merge_on_write = false;
+    std::set<int32_t> skip_inverted_index;
+    DataWriteType write_type = DataWriteType::TYPE_DEFAULT;
+    std::shared_ptr<Tablet> tablet = nullptr;
+    // for tracing local schema change record
+    std::shared_ptr<vectorized::schema_util::LocalSchemaChangeRecorder> schema_change_recorder =
+            nullptr;
+
+    std::shared_ptr<MowContext> mow_context;
 };
 
 } // namespace doris
