@@ -44,7 +44,10 @@
 #include "olap/data_dir.h"
 #include "olap/olap_common.h"
 #include "olap/olap_define.h"
+#include "olap/olap_meta.h"
+#include "olap/pb_helper.h"
 #include "olap/rowset/rowset.h"
+#include "olap/rowset/rowset_meta_manager.h"
 #include "olap/storage_engine.h"
 #include "olap/tablet.h"
 #include "olap/tablet_meta.h"
@@ -99,7 +102,10 @@ TabletManager::~TabletManager() {
 }
 
 Status TabletManager::_add_tablet_unlocked(TTabletId tablet_id, const TabletSharedPtr& tablet,
-                                           bool update_meta, bool force) {
+                                           bool update_meta, bool force, RuntimeProfile* profile) {
+    if (profile->get_counter("AddTablet") == nullptr) {
+        ADD_TIMER(profile, "AddTablet");
+    }
     Status res = Status::OK();
     VLOG_NOTICE << "begin to add tablet to TabletManager. "
                 << "tablet_id=" << tablet_id << ", force=" << force;
@@ -113,7 +119,7 @@ Status TabletManager::_add_tablet_unlocked(TTabletId tablet_id, const TabletShar
 
     if (existed_tablet == nullptr) {
         return _add_tablet_to_map_unlocked(tablet_id, tablet, update_meta, false /*keep_files*/,
-                                           false /*drop_old*/);
+                                           false /*drop_old*/, profile);
     }
     // During restore process, the tablet is exist and snapshot loader will replace the tablet's rowsets
     // and then reload the tablet, the tablet's path will the same
@@ -129,6 +135,9 @@ Status TabletManager::_add_tablet_unlocked(TTabletId tablet_id, const TabletShar
         }
     }
 
+    MonotonicStopWatch watch;
+    watch.start();
+
     // During storage migration, the tablet is moved to another disk, have to check
     // if the new tablet's rowset version is larger than the old one to prevent losting data during
     // migration
@@ -152,6 +161,8 @@ Status TabletManager::_add_tablet_unlocked(TTabletId tablet_id, const TabletShar
         old_version = old_rowset == nullptr ? -1 : old_rowset->end_version();
         new_version = new_rowset->end_version();
     }
+    COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "GetExistTabletVersion", "AddTablet"),
+                   static_cast<int64_t>(watch.reset()));
 
     // In restore process, we replace all origin files in tablet dir with
     // the downloaded snapshot files. Then we try to reload tablet header.
@@ -168,10 +179,12 @@ Status TabletManager::_add_tablet_unlocked(TTabletId tablet_id, const TabletShar
         (new_version > old_version || (new_version == old_version && new_time >= old_time))) {
         // check if new tablet's meta is in store and add new tablet's meta to meta store
         res = _add_tablet_to_map_unlocked(tablet_id, tablet, update_meta, keep_files,
-                                          true /*drop_old*/);
+                                          true /*drop_old*/, profile);
     } else {
         tablet->set_tablet_state(TABLET_SHUTDOWN);
         tablet->save_meta();
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "SaveMeta", "AddTablet"),
+                       static_cast<int64_t>(watch.reset()));
         {
             std::lock_guard<std::shared_mutex> shutdown_tablets_wrlock(_shutdown_tablets_lock);
             _shutdown_tablets.push_back(tablet);
@@ -193,22 +206,29 @@ Status TabletManager::_add_tablet_unlocked(TTabletId tablet_id, const TabletShar
 
 Status TabletManager::_add_tablet_to_map_unlocked(TTabletId tablet_id,
                                                   const TabletSharedPtr& tablet, bool update_meta,
-                                                  bool keep_files, bool drop_old) {
+                                                  bool keep_files, bool drop_old,
+                                                  RuntimeProfile* profile) {
     // check if new tablet's meta is in store and add new tablet's meta to meta store
     Status res = Status::OK();
+    MonotonicStopWatch watch;
+    watch.start();
     if (update_meta) {
         // call tablet save meta in order to valid the meta
         tablet->save_meta();
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "SaveMeta", "AddTablet"),
+                       static_cast<int64_t>(watch.reset()));
     }
     if (drop_old) {
         // If the new tablet is fresher than the existing one, then replace
         // the existing tablet with the new one.
         // Use default replica_id to ignore whether replica_id is match when drop tablet.
+        Status status = _drop_tablet_unlocked(tablet_id, /* replica_id */ 0, keep_files, false);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "DropOldTablet", "AddTablet"),
+                       static_cast<int64_t>(watch.reset()));
         RETURN_NOT_OK_STATUS_WITH_WARN(
-                _drop_tablet_unlocked(tablet_id, /* replica_id */ 0, keep_files, false),
-                strings::Substitute("failed to drop old tablet when add new tablet. "
-                                    "tablet_id=$0",
-                                    tablet_id));
+                status, strings::Substitute("failed to drop old tablet when add new tablet. "
+                                            "tablet_id=$0",
+                                            tablet_id));
     }
     // Register tablet into DataDir, so that we can manage tablet from
     // the perspective of root path.
@@ -221,6 +241,8 @@ Status TabletManager::_add_tablet_to_map_unlocked(TTabletId tablet_id,
     // Because table schema will copy in tablet, there will be double mem cost
     // so here multiply 2
     _tablet_meta_mem_tracker->consume(tablet->tablet_meta()->mem_size() * 2);
+    COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "RegisterTabletInfo", "AddTablet"),
+                   static_cast<int64_t>(watch.reset()));
 
     VLOG_NOTICE << "add tablet to map successfully."
                 << " tablet_id=" << tablet_id;
@@ -238,7 +260,8 @@ bool TabletManager::_check_tablet_id_exist_unlocked(TTabletId tablet_id) {
     return tablet_map.find(tablet_id) != tablet_map.end();
 }
 
-Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector<DataDir*> stores) {
+Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector<DataDir*> stores,
+                                    RuntimeProfile* profile) {
     SCOPED_CONSUME_MEM_TRACKER(_mem_tracker);
     DorisMetrics::instance()->create_tablet_requests_total->increment(1);
 
@@ -253,10 +276,16 @@ Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector
     bool need_two_lock = is_schema_change && ((_tablets_shards_mask & request.base_tablet_id) !=
                                               (_tablets_shards_mask & tablet_id));
     if (need_two_lock) {
+        SCOPED_TIMER(ADD_TIMER(profile, "GetTwoTableLock"));
         two_tablet_lock.lock();
     }
 
+    MonotonicStopWatch shard_lock_watch;
+    shard_lock_watch.start();
     std::lock_guard wrlock(_get_tablets_shard_lock(tablet_id));
+    shard_lock_watch.stop();
+    COUNTER_UPDATE(ADD_TIMER(profile, "GetShardLock"),
+                   static_cast<int64_t>(shard_lock_watch.elapsed_time()));
     // Make create_tablet operation to be idempotent:
     // 1. Return true if tablet with same tablet_id and schema_hash exist;
     //           false if tablet with same tablet_id but different schema_hash exist.
@@ -264,9 +293,12 @@ Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector
     // same) already exist, then just return true(an duplicate request). But if
     // tablet_id exist but with different schema_hash, return an error(report task will
     // eventually trigger its deletion).
-    if (_get_tablet_unlocked(tablet_id) != nullptr) {
-        LOG(INFO) << "success to create tablet. tablet already exist. tablet_id=" << tablet_id;
-        return Status::OK();
+    {
+        SCOPED_TIMER(ADD_TIMER(profile, "GetTabletUnlocked"));
+        if (_get_tablet_unlocked(tablet_id) != nullptr) {
+            LOG(INFO) << "success to create tablet. tablet already exist. tablet_id=" << tablet_id;
+            return Status::OK();
+        }
     }
 
     TabletSharedPtr base_tablet = nullptr;
@@ -274,9 +306,11 @@ Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector
     if (is_schema_change) {
         // if base_tablet_id's lock diffrent with new_tablet_id, we need lock it.
         if (need_two_lock) {
+            SCOPED_TIMER(ADD_TIMER(profile, "GetBaseTablet"));
             base_tablet = get_tablet(request.base_tablet_id);
             two_tablet_lock.unlock();
         } else {
+            SCOPED_TIMER(ADD_TIMER(profile, "GetBaseTabletUnlocked"));
             base_tablet = _get_tablet_unlocked(request.base_tablet_id);
         }
         if (base_tablet == nullptr) {
@@ -296,8 +330,8 @@ Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector
     }
 
     // set alter type to schema-change. it is useless
-    TabletSharedPtr tablet =
-            _internal_create_tablet_unlocked(request, is_schema_change, base_tablet.get(), stores);
+    TabletSharedPtr tablet = _internal_create_tablet_unlocked(request, is_schema_change,
+                                                              base_tablet.get(), stores, profile);
     if (tablet == nullptr) {
         DorisMetrics::instance()->create_tablet_requests_failed->increment(1);
         return Status::Error<CE_CMD_PARAMS_ERROR>("fail to create tablet. tablet_id={}",
@@ -310,7 +344,7 @@ Status TabletManager::create_tablet(const TCreateTabletReq& request, std::vector
 
 TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
         const TCreateTabletReq& request, const bool is_schema_change, const Tablet* base_tablet,
-        const std::vector<DataDir*>& data_dirs) {
+        const std::vector<DataDir*>& data_dirs, RuntimeProfile* profile) {
     // If in schema-change state, base_tablet must also be provided.
     // i.e., is_schema_change and base_tablet are either assigned or not assigned
     DCHECK((is_schema_change && base_tablet) || (!is_schema_change && !base_tablet));
@@ -318,8 +352,15 @@ TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
     // NOTE: The existence of tablet_id and schema_hash has already been checked,
     // no need check again here.
 
-    auto tablet =
-            _create_tablet_meta_and_dir_unlocked(request, is_schema_change, base_tablet, data_dirs);
+    const std::string parent_timer_name = "InternalCreateTablet";
+    SCOPED_TIMER(ADD_TIMER(profile, parent_timer_name));
+
+    MonotonicStopWatch watch;
+    watch.start();
+    auto create_meta_timer = ADD_CHILD_TIMER(profile, "CreateMeta", parent_timer_name);
+    auto tablet = _create_tablet_meta_and_dir_unlocked(request, is_schema_change, base_tablet,
+                                                       data_dirs, profile);
+    COUNTER_UPDATE(create_meta_timer, static_cast<int64_t>(watch.reset()));
     if (tablet == nullptr) {
         return nullptr;
     }
@@ -338,6 +379,8 @@ TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
     bool is_tablet_added = false;
     do {
         res = tablet->init();
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "TabletInit", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
         if (!res.ok()) {
             LOG(WARNING) << "tablet init failed. tablet:" << tablet->full_name();
             break;
@@ -348,6 +391,8 @@ TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
         // if (!in_restore_mode && request.__isset.version) {
         // create initial rowset before add it to storage engine could omit many locks
         res = tablet->create_initial_rowset(request.version);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "InitRowset", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
         if (!res.ok()) {
             LOG(WARNING) << "fail to create initial version for tablet. res=" << res;
             break;
@@ -361,7 +406,9 @@ TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
         }
         // Add tablet to StorageEngine will make it visible to user
         // Will persist tablet meta
-        res = _add_tablet_unlocked(new_tablet_id, tablet, /*update_meta*/ true, false);
+        auto add_tablet_timer = ADD_CHILD_TIMER(profile, "AddTablet", parent_timer_name);
+        res = _add_tablet_unlocked(new_tablet_id, tablet, /*update_meta*/ true, false, profile);
+        COUNTER_UPDATE(add_tablet_timer, static_cast<int64_t>(watch.reset()));
         if (!res.ok()) {
             LOG(WARNING) << "fail to add tablet to StorageEngine. res=" << res;
             break;
@@ -371,6 +418,8 @@ TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
         // TODO(lingbin): The following logic seems useless, can be removed?
         // Because if _add_tablet_unlocked() return OK, we must can get it from map.
         TabletSharedPtr tablet_ptr = _get_tablet_unlocked(new_tablet_id);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "GetTablet", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
         if (tablet_ptr == nullptr) {
             res = Status::Error<TABLE_NOT_FOUND>("fail to get tablet. res={}", res);
             break;
@@ -383,12 +432,16 @@ TabletSharedPtr TabletManager::_internal_create_tablet_unlocked(
     // something is wrong, we need clear environment
     if (is_tablet_added) {
         Status status = _drop_tablet_unlocked(new_tablet_id, request.replica_id, false, false);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "DropTablet", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
         if (!status.ok()) {
             LOG(WARNING) << "fail to drop tablet when create tablet failed. res=" << res;
         }
     } else {
         tablet->delete_all_files();
         TabletMetaManager::remove(data_dir, new_tablet_id, new_schema_hash);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "RemoveTabletFiles", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
     }
     return nullptr;
 }
@@ -403,21 +456,28 @@ static string _gen_tablet_dir(const string& dir, int16_t shard_id, int64_t table
 
 TabletSharedPtr TabletManager::_create_tablet_meta_and_dir_unlocked(
         const TCreateTabletReq& request, const bool is_schema_change, const Tablet* base_tablet,
-        const std::vector<DataDir*>& data_dirs) {
+        const std::vector<DataDir*>& data_dirs, RuntimeProfile* profile) {
     string pending_id = StrCat(TABLET_ID_PREFIX, request.tablet_id);
     // Many attempts are made here in the hope that even if a disk fails, it can still continue.
+    std::string parent_timer_name = "CreateMeta";
     DataDir* last_dir = nullptr;
+    MonotonicStopWatch watch;
+    watch.start();
     for (auto& data_dir : data_dirs) {
         if (last_dir != nullptr) {
             // If last_dir != null, it means the last attempt to create a tablet failed
             last_dir->remove_pending_ids(pending_id);
         }
         last_dir = data_dir;
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "RemovePendingIds", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
 
         TabletMetaSharedPtr tablet_meta;
         // if create meta failed, do not need to clean dir, because it is only in memory
         Status res = _create_tablet_meta_unlocked(request, data_dir, is_schema_change, base_tablet,
                                                   &tablet_meta);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "CreateMetaUnlock", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
         if (!res.ok()) {
             LOG(WARNING) << "fail to create tablet meta. res=" << res
                          << ", root=" << data_dir->path();
@@ -448,6 +508,8 @@ TabletSharedPtr TabletManager::_create_tablet_meta_and_dir_unlocked(
         }
 
         TabletSharedPtr new_tablet = Tablet::create_tablet_from_meta(tablet_meta, data_dir);
+        COUNTER_UPDATE(ADD_CHILD_TIMER(profile, "CreateTabletFromMeta", parent_timer_name),
+                       static_cast<int64_t>(watch.reset()));
         DCHECK(new_tablet != nullptr);
         return new_tablet;
     }
@@ -487,6 +549,18 @@ Status TabletManager::_drop_tablet_unlocked(TTabletId tablet_id, TReplicaId repl
     _remove_tablet_from_partition(to_drop_tablet);
     tablet_map_t& tablet_map = _get_tablet_map(tablet_id);
     tablet_map.erase(tablet_id);
+    {
+        std::shared_lock rlock(to_drop_tablet->get_header_lock());
+        static auto recycle_segment_cache = [](const auto& rowset_map) {
+            for (auto& [_, rowset] : rowset_map) {
+                // If the tablet was deleted, it need to remove all rowsets fds directly
+                SegmentLoader::instance()->erase_segments(
+                        SegmentCache::CacheKey(rowset->rowset_id()));
+            }
+        };
+        recycle_segment_cache(to_drop_tablet->rowset_map());
+        recycle_segment_cache(to_drop_tablet->stale_rowset_map());
+    }
     if (!keep_files) {
         // drop tablet will update tablet meta, should lock
         std::lock_guard<std::shared_mutex> wrlock(to_drop_tablet->get_header_lock());
@@ -836,9 +910,10 @@ Status TabletManager::load_tablet_from_meta(DataDir* data_dir, TTabletId tablet_
             tablet->init(),
             strings::Substitute("tablet init failed. tablet=$0", tablet->full_name()));
 
+    RuntimeProfile profile("CreateTablet");
     std::lock_guard<std::shared_mutex> wrlock(_get_tablets_shard_lock(tablet_id));
     RETURN_NOT_OK_STATUS_WITH_WARN(
-            _add_tablet_unlocked(tablet_id, tablet, update_meta, force),
+            _add_tablet_unlocked(tablet_id, tablet, update_meta, force, &profile),
             strings::Substitute("fail to add tablet. tablet=$0", tablet->full_name()));
 
     return Status::OK();
@@ -871,12 +946,59 @@ Status TabletManager::load_tablet_from_dir(DataDir* store, TTabletId tablet_id,
         return Status::Error<ENGINE_LOAD_INDEX_TABLE_ERROR>(
                 "fail to load tablet_meta. file_path={}", header_path);
     }
+    TabletUid tablet_uid = TabletUid::gen_uid();
+
+    // remove rowset binlog metas
+    auto binlog_metas_file = fmt::format("{}/rowset_binlog_metas.pb", schema_hash_path);
+    bool binlog_metas_file_exists = false;
+    auto file_exists_status =
+            io::global_local_filesystem()->exists(binlog_metas_file, &binlog_metas_file_exists);
+    if (!file_exists_status.ok()) {
+        return file_exists_status;
+    }
+    bool contain_binlog = false;
+    RowsetBinlogMetasPB rowset_binlog_metas_pb;
+    if (binlog_metas_file_exists) {
+        auto binlog_meta_filesize = std::filesystem::file_size(binlog_metas_file);
+        if (binlog_meta_filesize > 0) {
+            contain_binlog = true;
+            RETURN_IF_ERROR(read_pb(binlog_metas_file, &rowset_binlog_metas_pb));
+        }
+        RETURN_IF_ERROR(io::global_local_filesystem()->delete_file(binlog_metas_file));
+    }
+    if (contain_binlog) {
+        auto binlog_dir = fmt::format("{}/_binlog", schema_hash_path);
+        RETURN_IF_ERROR(io::global_local_filesystem()->create_directory(binlog_dir));
+
+        std::vector<io::FileInfo> files;
+        RETURN_IF_ERROR(
+                io::global_local_filesystem()->list(schema_hash_path, true, &files, &exists));
+        for (auto& file : files) {
+            auto& filename = file.file_name;
+            if (!filename.ends_with(".binlog")) {
+                continue;
+            }
+
+            // change clone_file suffix .binlog to .dat
+            std::string new_filename = filename;
+            new_filename.replace(filename.size() - 7, 7, ".dat");
+            auto from = fmt::format("{}/{}", schema_hash_path, filename);
+            auto to = fmt::format("{}/_binlog/{}", schema_hash_path, new_filename);
+            RETURN_IF_ERROR(io::global_local_filesystem()->rename(from, to));
+        }
+
+        auto meta = store->get_meta();
+        // if ingest binlog metas error, it will be gc in gc_unused_binlog_metas
+        RETURN_IF_ERROR(
+                RowsetMetaManager::ingest_binlog_metas(meta, tablet_uid, &rowset_binlog_metas_pb));
+    }
+
     // has to change shard id here, because meta file maybe copied from other source
     // its shard is different from local shard
     tablet_meta->set_shard_id(shard);
     // load dir is called by clone, restore, storage migration
     // should change tablet uid when tablet object changed
-    tablet_meta->set_tablet_uid(TabletUid::gen_uid());
+    tablet_meta->set_tablet_uid(std::move(tablet_uid));
     std::string meta_binary;
     tablet_meta->serialize(&meta_binary);
     RETURN_NOT_OK_STATUS_WITH_WARN(
