@@ -17,25 +17,46 @@
 
 #include "runtime/snapshot_loader.h"
 
+// IWYU pragma: no_include <bthread/errno.h>
+#include <errno.h> // IWYU pragma: keep
+#include <fmt/format.h>
+#include <gen_cpp/FrontendService.h>
+#include <gen_cpp/FrontendService_types.h>
+#include <gen_cpp/HeartbeatService_types.h>
+#include <gen_cpp/PlanNodes_types.h>
+#include <gen_cpp/Status_types.h>
+#include <gen_cpp/Types_types.h>
 #include <stdint.h>
+#include <stdio.h>
+#include <unistd.h>
+
+#include <algorithm>
+#include <cstring>
+#include <filesystem>
+#include <istream>
+#include <unordered_map>
+#include <utility>
 
 #include "common/logging.h"
-#include "env/env.h"
-#include "gen_cpp/FrontendService.h"
-#include "gen_cpp/FrontendService_types.h"
-#include "gen_cpp/HeartbeatService_types.h"
-#include "gen_cpp/PaloBrokerService_types.h"
-#include "gen_cpp/TPaloBrokerService.h"
-#include "olap/file_helper.h"
+#include "gutil/strings/split.h"
+#include "http/http_client.h"
+#include "io/fs/broker_file_system.h"
+#include "io/fs/file_system.h"
+#include "io/fs/hdfs_file_system.h"
+#include "io/fs/local_file_system.h"
+#include "io/fs/path.h"
+#include "io/fs/remote_file_system.h"
+#include "io/fs/s3_file_system.h"
+#include "io/hdfs_builder.h"
+#include "olap/data_dir.h"
 #include "olap/snapshot_manager.h"
 #include "olap/storage_engine.h"
 #include "olap/tablet.h"
-#include "runtime/broker_mgr.h"
+#include "olap/tablet_manager.h"
+#include "runtime/client_cache.h"
 #include "runtime/exec_env.h"
-#include "util/broker_storage_backend.h"
-#include "util/file_utils.h"
-#include "util/hdfs_storage_backend.h"
-#include "util/s3_storage_backend.h"
+#include "util/s3_uri.h"
+#include "util/s3_util.h"
 #include "util/thrift_rpc_helper.h"
 
 namespace doris {
@@ -46,29 +67,42 @@ SnapshotLoader::SnapshotLoader(ExecEnv* env, int64_t job_id, int64_t task_id)
           _task_id(task_id),
           _broker_addr(TNetworkAddress()),
           _prop(std::map<std::string, std::string>()),
-          _storage_backend(nullptr) {}
+          _remote_fs(nullptr) {}
 
 SnapshotLoader::SnapshotLoader(ExecEnv* env, int64_t job_id, int64_t task_id,
                                const TNetworkAddress& broker_addr,
-                               const std::map<std::string, std::string>& prop,
-                               TStorageBackendType::type type)
-        : _env(env), _job_id(job_id), _task_id(task_id), _broker_addr(broker_addr), _prop(prop) {
+                               const std::map<std::string, std::string>& prop)
+        : _env(env), _job_id(job_id), _task_id(task_id), _broker_addr(broker_addr), _prop(prop) {}
+
+Status SnapshotLoader::init(TStorageBackendType::type type, const std::string& location) {
     if (TStorageBackendType::type::S3 == type) {
-        _storage_backend.reset(new S3StorageBackend(_prop));
+        S3Conf s3_conf;
+        S3URI s3_uri(location);
+        RETURN_IF_ERROR(s3_uri.parse());
+        RETURN_IF_ERROR(S3ClientFactory::convert_properties_to_s3_conf(_prop, s3_uri, &s3_conf));
+        std::shared_ptr<io::S3FileSystem> fs;
+        RETURN_IF_ERROR(io::S3FileSystem::create(std::move(s3_conf), "", &fs));
+        _remote_fs = std::move(fs);
     } else if (TStorageBackendType::type::HDFS == type) {
-        _storage_backend.reset(new HDFSStorageBackend(_prop));
+        THdfsParams hdfs_params = parse_properties(_prop);
+        std::shared_ptr<io::HdfsFileSystem> fs;
+        RETURN_IF_ERROR(io::HdfsFileSystem::create(hdfs_params, "", nullptr, &fs));
+        _remote_fs = std::move(fs);
     } else if (TStorageBackendType::type::BROKER == type) {
-        _storage_backend.reset(new BrokerStorageBackend(_env, _broker_addr, _prop));
+        std::shared_ptr<io::BrokerFileSystem> fs;
+        RETURN_IF_ERROR(io::BrokerFileSystem::create(_broker_addr, _prop, &fs));
+        _remote_fs = std::move(fs);
     } else {
-        _storage_backend = nullptr;
+        return Status::InternalError("Unknown storage tpye: {}", type);
     }
+    return Status::OK();
 }
 
 SnapshotLoader::~SnapshotLoader() = default;
 
 Status SnapshotLoader::upload(const std::map<std::string, std::string>& src_to_dest_path,
                               std::map<int64_t, std::vector<std::string>>* tablet_files) {
-    if (!_storage_backend) {
+    if (!_remote_fs) {
         return Status::InternalError("Storage backend not initialized.");
     }
     LOG(INFO) << "begin to upload snapshot files. num: " << src_to_dest_path.size()
@@ -99,7 +133,7 @@ Status SnapshotLoader::upload(const std::map<std::string, std::string>& src_to_d
 
         // 2.1 get existing files from remote path
         std::map<std::string, FileStat> remote_files;
-        RETURN_IF_ERROR(_storage_backend->list(dest_path, true, false, &remote_files));
+        RETURN_IF_ERROR(_list_with_checksum(dest_path, &remote_files));
 
         for (auto& tmp : remote_files) {
             VLOG_CRITICAL << "get remote file: " << tmp.first << ", checksum: " << tmp.second.md5;
@@ -118,13 +152,8 @@ Status SnapshotLoader::upload(const std::map<std::string, std::string>& src_to_d
             const std::string& local_file = *it;
             // calc md5sum of localfile
             std::string md5sum;
-            status = FileUtils::md5sum(src_path + "/" + local_file, &md5sum);
-            if (!status.ok()) {
-                std::stringstream ss;
-                ss << "failed to get md5sum of file: " << local_file << ": " << status;
-                LOG(WARNING) << ss.str();
-                return Status::InternalError(ss.str());
-            }
+            RETURN_IF_ERROR(
+                    io::global_local_filesystem()->md5sum(src_path + "/" + local_file, &md5sum));
             VLOG_CRITICAL << "get file checksum: " << local_file << ": " << md5sum;
             local_files_with_checksum.push_back(local_file + "." + md5sum);
 
@@ -151,8 +180,8 @@ Status SnapshotLoader::upload(const std::map<std::string, std::string>& src_to_d
             // upload
             std::string full_remote_file = dest_path + "/" + local_file;
             std::string full_local_file = src_path + "/" + local_file;
-            RETURN_IF_ERROR(_storage_backend->upload_with_checksum(full_local_file,
-                                                                   full_remote_file, md5sum));
+            RETURN_IF_ERROR(
+                    _remote_fs->upload_with_checksum(full_local_file, full_remote_file, md5sum));
         } // end for each tablet's local files
 
         tablet_files->emplace(tablet_id, local_files_with_checksum);
@@ -172,7 +201,7 @@ Status SnapshotLoader::upload(const std::map<std::string, std::string>& src_to_d
  */
 Status SnapshotLoader::download(const std::map<std::string, std::string>& src_to_dest_path,
                                 std::vector<int64_t>* downloaded_tablet_ids) {
-    if (!_storage_backend) {
+    if (!_remote_fs) {
         return Status::InternalError("Storage backend not initialized.");
     }
     LOG(INFO) << "begin to download snapshot files. num: " << src_to_dest_path.size()
@@ -213,7 +242,7 @@ Status SnapshotLoader::download(const std::map<std::string, std::string>& src_to
 
         // 2.2. get remote files
         std::map<std::string, FileStat> remote_files;
-        RETURN_IF_ERROR(_storage_backend->list(remote_path, true, false, &remote_files));
+        RETURN_IF_ERROR(_list_with_checksum(remote_path, &remote_files));
         if (remote_files.empty()) {
             std::stringstream ss;
             ss << "get nothing from remote path: " << remote_path;
@@ -249,7 +278,8 @@ Status SnapshotLoader::download(const std::map<std::string, std::string>& src_to
                 } else {
                     // check checksum
                     std::string local_md5sum;
-                    Status st = FileUtils::md5sum(local_path + "/" + remote_file, &local_md5sum);
+                    Status st = io::global_local_filesystem()->md5sum(
+                            local_path + "/" + remote_file, &local_md5sum);
                     if (!st.ok()) {
                         LOG(WARNING) << "failed to get md5sum of local file: " << remote_file
                                      << ". msg: " << st << ". download it";
@@ -287,17 +317,12 @@ Status SnapshotLoader::download(const std::map<std::string, std::string>& src_to
             // remove file which will be downloaded now.
             // this file will be added to local_files if it be downloaded successfully.
             local_files.erase(find);
-            RETURN_IF_ERROR(_storage_backend->download(full_remote_file, full_local_file));
+            RETURN_IF_ERROR(_remote_fs->download(full_remote_file, full_local_file));
 
             // 3. check md5 of the downloaded file
             std::string downloaded_md5sum;
-            status = FileUtils::md5sum(full_local_file, &downloaded_md5sum);
-            if (!status.ok()) {
-                std::stringstream ss;
-                ss << "failed to get md5sum of file: " << full_local_file << ", err: " << status;
-                LOG(WARNING) << ss.str();
-                return Status::InternalError(ss.str());
-            }
+            RETURN_IF_ERROR(
+                    io::global_local_filesystem()->md5sum(full_local_file, &downloaded_md5sum));
             VLOG_CRITICAL << "get downloaded file checksum: " << full_local_file << ": "
                           << downloaded_md5sum;
             if (downloaded_md5sum != file_stat.md5) {
@@ -348,6 +373,272 @@ Status SnapshotLoader::download(const std::map<std::string, std::string>& src_to
     return status;
 }
 
+Status SnapshotLoader::remote_http_download(
+        const std::vector<TRemoteTabletSnapshot>& remote_tablet_snapshots,
+        std::vector<int64_t>* downloaded_tablet_ids) {
+    LOG(INFO) << fmt::format("begin to download snapshots via http. job: {}, task id: {}", _job_id,
+                             _task_id);
+    constexpr uint32_t kListRemoteFileTimeout = 15;
+    constexpr uint32_t kDownloadFileMaxRetry = 3;
+    constexpr uint32_t kGetLengthTimeout = 10;
+
+    // check if job has already been cancelled
+    int tmp_counter = 1;
+    RETURN_IF_ERROR(_report_every(0, &tmp_counter, 0, 0, TTaskType::type::DOWNLOAD));
+    Status status = Status::OK();
+
+    // Step before, validate all remote
+
+    // Step 1: Validate local tablet snapshot paths
+    for (auto& remote_tablet_snapshot : remote_tablet_snapshots) {
+        auto& path = remote_tablet_snapshot.local_snapshot_path;
+        bool res = true;
+        RETURN_IF_ERROR(io::global_local_filesystem()->is_directory(path, &res));
+        if (!res) {
+            std::stringstream ss;
+            auto err_msg =
+                    fmt::format("snapshot path is not directory or does not exist: {}", path);
+            LOG(WARNING) << err_msg;
+            return Status::RuntimeError(err_msg);
+        }
+    }
+
+    // Step 2: get all local files
+    struct LocalFileStat {
+        uint64_t size;
+        // TODO(Drogon): add md5sum
+    };
+    std::unordered_map<std::string, std::unordered_map<std::string, LocalFileStat>> local_files_map;
+    for (auto& remote_tablet_snapshot : remote_tablet_snapshots) {
+        const auto& local_path = remote_tablet_snapshot.local_snapshot_path;
+        std::vector<std::string> local_files;
+        RETURN_IF_ERROR(_get_existing_files_from_local(local_path, &local_files));
+
+        auto& local_filestat = local_files_map[local_path];
+        for (auto& local_file : local_files) {
+            // add file size
+            std::string local_file_path = local_path + "/" + local_file;
+            std::error_code ec;
+            uint64_t local_file_size = std::filesystem::file_size(local_file_path, ec);
+            if (ec) {
+                LOG(WARNING) << "download file error" << ec.message();
+                return Status::IOError("can't retrive file_size of {}, due to {}", local_file_path,
+                                       ec.message());
+            }
+            local_filestat[local_file] = {local_file_size};
+        }
+    }
+
+    // Step 3: Validate remote tablet snapshot paths && remote files map
+    // TODO(Drogon): Add md5sum check
+    // key is remote snapshot paths, value is filelist
+    // get all these use http download action
+    // http://172.16.0.14:6781/api/_tablet/_download?token=e804dd27-86da-4072-af58-70724075d2a4&file=/home/ubuntu/doris_master/output/be/storage/snapshot/20230410102306.9.180//2774718/217609978/2774718.hdr
+    int report_counter = 0;
+    int total_num = remote_tablet_snapshots.size();
+    int finished_num = 0;
+    struct RemoteFileStat {
+        // TODO(Drogon): Add md5sum
+        std::string url;
+        uint64_t size;
+    };
+    std::unordered_map<std::string, std::unordered_map<std::string, RemoteFileStat>>
+            remote_files_map;
+    for (auto& remote_tablet_snapshot : remote_tablet_snapshots) {
+        const auto& remote_path = remote_tablet_snapshot.remote_snapshot_path;
+        auto& remote_files = remote_files_map[remote_path];
+        const auto& token = remote_tablet_snapshot.remote_token;
+        const auto& remote_be_addr = remote_tablet_snapshot.remote_be_addr;
+
+        // HEAD http://172.16.0.14:6781/api/_tablet/_download?token=e804dd27-86da-4072-af58-70724075d2a4&file=/home/ubuntu/doris_master/output/be/storage/snapshot/20230410102306.9.180/
+        std::string remote_url_prefix =
+                fmt::format("http://{}:{}/api/_tablet/_download?token={}&file={}",
+                            remote_be_addr.hostname, remote_be_addr.port, token, remote_path);
+
+        string file_list_str;
+        auto list_files_cb = [&remote_url_prefix, &file_list_str](HttpClient* client) {
+            RETURN_IF_ERROR(client->init(remote_url_prefix));
+            client->set_timeout_ms(kListRemoteFileTimeout * 1000);
+            return client->execute(&file_list_str);
+        };
+        RETURN_IF_ERROR(HttpClient::execute_with_retry(kDownloadFileMaxRetry, 1, list_files_cb));
+        std::vector<string> filename_list =
+                strings::Split(file_list_str, "\n", strings::SkipWhitespace());
+
+        for (const auto& filename : filename_list) {
+            std::string remote_file_url = fmt::format(
+                    "http://{}:{}/api/_tablet/_download?token={}&file={}/{}",
+                    remote_tablet_snapshot.remote_be_addr.hostname,
+                    remote_tablet_snapshot.remote_be_addr.port, remote_tablet_snapshot.remote_token,
+                    remote_tablet_snapshot.remote_snapshot_path, filename);
+
+            // get file length
+            uint64_t file_size = 0;
+            auto get_file_size_cb = [&remote_file_url, &file_size](HttpClient* client) {
+                RETURN_IF_ERROR(client->init(remote_file_url));
+                client->set_timeout_ms(kGetLengthTimeout * 1000);
+                RETURN_IF_ERROR(client->head());
+                RETURN_IF_ERROR(client->get_content_length(&file_size));
+                return Status::OK();
+            };
+            RETURN_IF_ERROR(
+                    HttpClient::execute_with_retry(kDownloadFileMaxRetry, 1, get_file_size_cb));
+
+            remote_files[filename] = RemoteFileStat {remote_file_url, file_size};
+        }
+    }
+
+    // Step 4: Compare local and remote files && get all need download files
+    for (auto& remote_tablet_snapshot : remote_tablet_snapshots) {
+        RETURN_IF_ERROR(_report_every(10, &report_counter, finished_num, total_num,
+                                      TTaskType::type::DOWNLOAD));
+
+        const auto& remote_path = remote_tablet_snapshot.remote_snapshot_path;
+        const auto& local_path = remote_tablet_snapshot.local_snapshot_path;
+        auto& remote_files = remote_files_map[remote_path];
+        auto& local_files = local_files_map[local_path];
+        auto remote_tablet_id = remote_tablet_snapshot.remote_tablet_id;
+
+        // get all need download files
+        std::vector<std::string> need_download_files;
+        for (const auto& [remote_file, remote_filestat] : remote_files) {
+            LOG(INFO) << fmt::format("remote file: {}, size: {}", remote_file,
+                                     remote_filestat.size);
+            auto it = local_files.find(remote_file);
+            if (it == local_files.end()) {
+                need_download_files.emplace_back(remote_file);
+                continue;
+            }
+            if (_end_with(remote_file, ".hdr")) {
+                need_download_files.emplace_back(remote_file);
+                continue;
+            }
+
+            if (auto& local_filestat = it->second; local_filestat.size != remote_filestat.size) {
+                need_download_files.emplace_back(remote_file);
+                continue;
+            }
+            // TODO(Drogon): check by md5sum, if not match then download
+
+            LOG(INFO) << fmt::format("file {} already exists, skip download", remote_file);
+        }
+
+        auto local_tablet_id = remote_tablet_snapshot.local_tablet_id;
+        TabletSharedPtr tablet =
+                _env->storage_engine()->tablet_manager()->get_tablet(local_tablet_id);
+        if (tablet == nullptr) {
+            std::stringstream ss;
+            ss << "failed to get local tablet: " << local_tablet_id;
+            LOG(WARNING) << ss.str();
+            return Status::InternalError(ss.str());
+        }
+        DataDir* data_dir = tablet->data_dir();
+
+        // download all need download files
+        uint64_t total_file_size = 0;
+        MonotonicStopWatch watch;
+        watch.start();
+        for (auto& filename : need_download_files) {
+            auto& remote_filestat = remote_files[filename];
+            auto file_size = remote_filestat.size;
+            auto& remote_file_url = remote_filestat.url;
+
+            // check disk capacity
+            if (data_dir->reach_capacity_limit(file_size)) {
+                return Status::InternalError("Disk reach capacity limit");
+            }
+
+            total_file_size += file_size;
+            uint64_t estimate_timeout = file_size / config::download_low_speed_limit_kbps / 1024;
+            if (estimate_timeout < config::download_low_speed_time) {
+                estimate_timeout = config::download_low_speed_time;
+            }
+
+            std::string local_filename;
+            RETURN_IF_ERROR(_replace_tablet_id(filename, local_tablet_id, &local_filename));
+            std::string local_file_path = local_path + "/" + local_filename;
+
+            LOG(INFO) << "clone begin to download file from: " << remote_file_url
+                      << " to: " << local_file_path << ". size(B): " << file_size
+                      << ", timeout(s): " << estimate_timeout;
+
+            auto download_cb = [&remote_file_url, estimate_timeout, &local_file_path,
+                                file_size](HttpClient* client) {
+                RETURN_IF_ERROR(client->init(remote_file_url));
+                client->set_timeout_ms(estimate_timeout * 1000);
+                RETURN_IF_ERROR(client->download(local_file_path));
+
+                std::error_code ec;
+                // Check file length
+                uint64_t local_file_size = std::filesystem::file_size(local_file_path, ec);
+                if (ec) {
+                    LOG(WARNING) << "download file error" << ec.message();
+                    return Status::IOError("can't retrive file_size of {}, due to {}",
+                                           local_file_path, ec.message());
+                }
+                if (local_file_size != file_size) {
+                    LOG(WARNING) << "download file length error"
+                                 << ", remote_path=" << remote_file_url
+                                 << ", file_size=" << file_size
+                                 << ", local_file_size=" << local_file_size;
+                    return Status::InternalError("downloaded file size is not equal");
+                }
+                chmod(local_file_path.c_str(), S_IRUSR | S_IWUSR);
+                return Status::OK();
+            };
+            RETURN_IF_ERROR(HttpClient::execute_with_retry(kDownloadFileMaxRetry, 1, download_cb));
+
+            // local_files always keep the updated local files
+            local_files[filename] = LocalFileStat {file_size};
+        }
+
+        uint64_t total_time_ms = watch.elapsed_time() / 1000 / 1000;
+        total_time_ms = total_time_ms > 0 ? total_time_ms : 0;
+        double copy_rate = 0.0;
+        if (total_time_ms > 0) {
+            copy_rate = total_file_size / ((double)total_time_ms) / 1000;
+        }
+        LOG(INFO) << fmt::format(
+                "succeed to copy remote tablet {} to local tablet {}, total file size: {} B, cost: "
+                "{} ms, rate: {} MB/s",
+                remote_tablet_id, local_tablet_id, total_file_size, total_time_ms, copy_rate);
+
+        // local_files: contain all remote files and local files
+        // finally, delete local files which are not in remote
+        for (const auto& [local_file, local_filestat] : local_files) {
+            // replace the tablet id in local file name with the remote tablet id,
+            // in order to compare the file name.
+            std::string new_name;
+            Status st = _replace_tablet_id(local_file, remote_tablet_id, &new_name);
+            if (!st.ok()) {
+                LOG(WARNING) << "failed to replace tablet id. unknown local file: " << st
+                             << ". ignore it";
+                continue;
+            }
+            VLOG_CRITICAL << "new file name after replace tablet id: " << new_name;
+            const auto& find = remote_files.find(new_name);
+            if (find != remote_files.end()) {
+                continue;
+            }
+
+            // delete
+            std::string full_local_file = local_path + "/" + local_file;
+            LOG(INFO) << "begin to delete local snapshot file: " << full_local_file
+                      << ", it does not exist in remote";
+            if (remove(full_local_file.c_str()) != 0) {
+                LOG(WARNING) << "failed to delete unknown local file: " << full_local_file
+                             << ", error: " << strerror(errno)
+                             << ", file size: " << local_filestat.size << ", ignore it";
+            }
+        }
+
+        ++finished_num;
+    }
+
+    LOG(INFO) << "finished to download snapshots. job: " << _job_id << ", task id: " << _task_id;
+    return status;
+}
+
 // move the snapshot files in snapshot_path
 // to tablet_path
 // If overwrite, just replace the tablet_path with snapshot_path,
@@ -517,6 +808,7 @@ Status SnapshotLoader::_get_tablet_id_and_schema_hash_from_file_path(const std::
 
 Status SnapshotLoader::_check_local_snapshot_paths(
         const std::map<std::string, std::string>& src_to_dest_path, bool check_src) {
+    bool res = true;
     for (const auto& pair : src_to_dest_path) {
         std::string path;
         if (check_src) {
@@ -524,7 +816,9 @@ Status SnapshotLoader::_check_local_snapshot_paths(
         } else {
             path = pair.second;
         }
-        if (!FileUtils::is_dir(path)) {
+
+        RETURN_IF_ERROR(io::global_local_filesystem()->is_directory(path, &res));
+        if (!res) {
             std::stringstream ss;
             ss << "snapshot path is not directory or does not exist: " << path;
             LOG(WARNING) << ss.str();
@@ -537,12 +831,11 @@ Status SnapshotLoader::_check_local_snapshot_paths(
 
 Status SnapshotLoader::_get_existing_files_from_local(const std::string& local_path,
                                                       std::vector<std::string>* local_files) {
-    Status status = FileUtils::list_files(Env::Default(), local_path, local_files);
-    if (!status.ok()) {
-        std::stringstream ss;
-        ss << "failed to list files in local path: " << local_path << ", msg: " << status;
-        LOG(WARNING) << ss.str();
-        return status;
+    bool exists = true;
+    std::vector<io::FileInfo> files;
+    RETURN_IF_ERROR(io::global_local_filesystem()->list(local_path, true, &files, &exists));
+    for (auto& file : files) {
+        local_files->push_back(file.file_name);
     }
     LOG(INFO) << "finished to list files in local path: " << local_path
               << ", file num: " << local_files->size();
@@ -628,4 +921,25 @@ Status SnapshotLoader::_report_every(int report_threshold, int* counter, int32_t
     return Status::OK();
 }
 
+Status SnapshotLoader::_list_with_checksum(const std::string& dir,
+                                           std::map<std::string, FileStat>* md5_files) {
+    bool exists = true;
+    std::vector<io::FileInfo> files;
+    RETURN_IF_ERROR(_remote_fs->list(dir, true, &files, &exists));
+    for (auto& tmp_file : files) {
+        io::Path path(tmp_file.file_name);
+        std::string file_name = path.filename();
+        size_t pos = file_name.find_last_of(".");
+        if (pos == std::string::npos || pos == file_name.size() - 1) {
+            // Not found checksum separator, ignore this file
+            continue;
+        }
+        FileStat stat = {std::string(file_name, 0, pos), std::string(file_name, pos + 1),
+                         tmp_file.file_size};
+        md5_files->emplace(std::string(file_name, 0, pos), stat);
+    }
+
+    return Status::OK();
+}
+
 } // end namespace doris
