@@ -17,24 +17,40 @@
 
 #include "util/thrift_server.h"
 
-#include <thrift/concurrency/Thread.h>
+#include <glog/logging.h>
+#include <thrift/Thrift.h>
 #include <thrift/concurrency/ThreadFactory.h>
 #include <thrift/concurrency/ThreadManager.h>
 #include <thrift/protocol/TBinaryProtocol.h>
+#include <thrift/protocol/TProtocol.h>
 #include <thrift/server/TNonblockingServer.h>
 #include <thrift/server/TThreadPoolServer.h>
 #include <thrift/server/TThreadedServer.h>
+#include <thrift/transport/TBufferTransports.h>
 #include <thrift/transport/TNonblockingServerSocket.h>
 #include <thrift/transport/TServerSocket.h>
 #include <thrift/transport/TSocket.h>
-
+#include <thrift/transport/TTransport.h>
+// IWYU pragma: no_include <bits/chrono.h>
+#include <chrono> // IWYU pragma: keep
 #include <condition_variable>
 #include <mutex>
 #include <sstream>
 #include <thread>
 
+#include "service/backend_options.h"
 #include "util/doris_metrics.h"
 
+namespace apache {
+namespace thrift {
+class TProcessor;
+
+namespace transport {
+class TServerTransport;
+} // namespace transport
+} // namespace thrift
+} // namespace apache
+
 namespace doris {
 
 DEFINE_GAUGE_METRIC_PROTOTYPE_3ARG(thrift_current_connections, MetricUnit::CONNECTIONS,
@@ -151,6 +167,8 @@ void ThriftServer::ThriftServerEventProcessor::supervise() {
                    << ") exited due to TException: " << e.what();
     }
 
+    LOG(INFO) << "ThriftServer " << _thrift_server->_name << " exited";
+
     {
         // _signal_lock ensures mutual exclusion of access to _thrift_server
         std::lock_guard<std::mutex> lock(_signal_lock);
@@ -285,11 +303,6 @@ Status ThriftServer::start() {
     DCHECK(!_started);
     std::shared_ptr<apache::thrift::protocol::TProtocolFactory> protocol_factory(
             new apache::thrift::protocol::TBinaryProtocolFactory());
-    // binary_protocal_factory for setStringSizeLimit function
-    std::shared_ptr<apache::thrift::protocol::TBinaryProtocolFactory> binary_protocal_factory =
-            std::dynamic_pointer_cast<apache::thrift::protocol::TBinaryProtocolFactory>(
-                    protocol_factory);
-    binary_protocal_factory->setStringSizeLimit(config::be_thrift_max_pkg_bytes);
     std::shared_ptr<apache::thrift::concurrency::ThreadManager> thread_mgr;
     std::shared_ptr<apache::thrift::concurrency::ThreadFactory> thread_factory =
             std::make_shared<apache::thrift::concurrency::ThreadFactory>();
@@ -320,7 +333,8 @@ Status ThriftServer::start() {
         break;
 
     case THREAD_POOL:
-        fe_server_transport.reset(new apache::thrift::transport::TServerSocket(_port));
+        fe_server_transport.reset(new apache::thrift::transport::TServerSocket(
+                BackendOptions::get_service_bind_address_without_bracket(), _port));
 
         if (transport_factory.get() == nullptr) {
             transport_factory.reset(new apache::thrift::transport::TBufferedTransportFactory());
@@ -331,7 +345,8 @@ Status ThriftServer::start() {
         break;
 
     case THREADED:
-        server_socket = new apache::thrift::transport::TServerSocket(_port);
+        server_socket = new apache::thrift::transport::TServerSocket(
+                BackendOptions::get_service_bind_address_without_bracket(), _port);
         //      server_socket->setAcceptTimeout(500);
         fe_server_transport.reset(server_socket);
 
@@ -369,7 +384,6 @@ void ThriftServer::stop() {
 
 void ThriftServer::join() {
     DCHECK(_server_thread != nullptr);
-    DCHECK(_started);
     _server_thread->join();
 }
 
