@@ -32,18 +32,10 @@
 
 namespace doris::vectorized {
 
-/// This class implements a wrapper around an aggregate function. Despite its name,
-/// this is an adapter. It is used to handle aggregate functions that are called with
-/// at least one nullable argument. It implements the logic according to which any
-/// row that contains at least one NULL is skipped.
-
-/// If all rows had NULL, the behaviour is determined by "result_is_nullable" template parameter.
-///  true - return NULL; false - return value from empty aggregation state of nested function.
-
-template <bool result_is_nullable, typename Derived>
-class AggregateFunctionNullBase : public IAggregateFunctionHelper<Derived> {
+template <typename NestFunction, bool result_is_nullable, typename Derived>
+class AggregateFunctionNullBaseInline : public IAggregateFunctionHelper<Derived> {
 protected:
-    AggregateFunctionPtr nested_function;
+    std::unique_ptr<NestFunction> nested_function;
     size_t prefix_size;
 
     /** In addition to data for nested aggregate function, we keep a flag
@@ -78,10 +70,10 @@ protected:
     }
 
 public:
-    AggregateFunctionNullBase(AggregateFunctionPtr nested_function_, const DataTypes& arguments,
-                              const Array& params)
-            : IAggregateFunctionHelper<Derived>(arguments, params),
-              nested_function {nested_function_} {
+    AggregateFunctionNullBaseInline(IAggregateFunction* nested_function_,
+                                    const DataTypes& arguments)
+            : IAggregateFunctionHelper<Derived>(arguments),
+              nested_function {assert_cast<NestFunction*>(nested_function_)} {
         if (result_is_nullable) {
             prefix_size = nested_function->align_of_data();
         } else {
@@ -151,25 +143,17 @@ public:
         }
     }
 
-    void deserialize_and_merge(AggregateDataPtr __restrict place, BufferReadable& buf,
-                               Arena* arena) const override {
+    void deserialize_and_merge(AggregateDataPtr __restrict place, AggregateDataPtr __restrict rhs,
+                               BufferReadable& buf, Arena* arena) const override {
         bool flag = true;
         if (result_is_nullable) {
             read_binary(flag, buf);
         }
         if (flag) {
+            set_flag(rhs);
             set_flag(place);
-            nested_function->deserialize_and_merge(nested_place(place), buf, arena);
-        }
-    }
-
-    void deserialize_and_merge_from_column(AggregateDataPtr __restrict place, const IColumn& column,
-                                           Arena* arena) const override {
-        size_t num_rows = column.size();
-        for (size_t i = 0; i != num_rows; ++i) {
-            VectorBufferReader buffer_reader(
-                    (assert_cast<const ColumnString&>(column)).get_data_at(i));
-            deserialize_and_merge(place, buffer_reader, arena);
+            nested_function->deserialize_and_merge(nested_place(place), nested_place(rhs), buf,
+                                                   arena);
         }
     }
 
@@ -198,16 +182,18 @@ public:
 /** There are two cases: for single argument and variadic.
   * Code for single argument is much more efficient.
   */
-template <bool result_is_nullable>
-class AggregateFunctionNullUnary final
-        : public AggregateFunctionNullBase<result_is_nullable,
-                                           AggregateFunctionNullUnary<result_is_nullable>> {
+template <typename NestFuction, bool result_is_nullable>
+class AggregateFunctionNullUnaryInline final
+        : public AggregateFunctionNullBaseInline<
+                  NestFuction, result_is_nullable,
+                  AggregateFunctionNullUnaryInline<NestFuction, result_is_nullable>> {
 public:
-    AggregateFunctionNullUnary(AggregateFunctionPtr nested_function_, const DataTypes& arguments,
-                               const Array& params)
-            : AggregateFunctionNullBase<result_is_nullable,
-                                        AggregateFunctionNullUnary<result_is_nullable>>(
-                      std::move(nested_function_), arguments, params) {}
+    AggregateFunctionNullUnaryInline(IAggregateFunction* nested_function_,
+                                     const DataTypes& arguments)
+            : AggregateFunctionNullBaseInline<
+                      NestFuction, result_is_nullable,
+                      AggregateFunctionNullUnaryInline<NestFuction, result_is_nullable>>(
+                      nested_function_, arguments) {}
 
     void add(AggregateDataPtr __restrict place, const IColumn** columns, size_t row_num,
              Arena* arena) const override {
@@ -219,6 +205,21 @@ public:
         }
     }
 
+    void add_batch(size_t batch_size, AggregateDataPtr* places, size_t place_offset,
+                   const IColumn** columns, Arena* arena, bool agg_many) const override {
+        const ColumnNullable* column = assert_cast<const ColumnNullable*>(columns[0]);
+        // The overhead introduced is negligible here, just an extra memory read from NullMap
+        const auto* __restrict null_map_data = column->get_null_map_data().data();
+        const IColumn* nested_column = &column->get_nested_column();
+        for (int i = 0; i < batch_size; ++i) {
+            if (!null_map_data[i]) {
+                AggregateDataPtr __restrict place = places[i] + place_offset;
+                this->set_flag(place);
+                this->nested_function->add(this->nested_place(place), &nested_column, i, arena);
+            }
+        }
+    }
+
     void add_batch_single_place(size_t batch_size, AggregateDataPtr place, const IColumn** columns,
                                 Arena* arena) const override {
         const ColumnNullable* column = assert_cast<const ColumnNullable*>(columns[0]);
@@ -226,10 +227,7 @@ public:
 
         if (has_null) {
             for (size_t i = 0; i < batch_size; ++i) {
-                if (!column->is_null_at(i)) {
-                    this->set_flag(place);
-                    this->add(place, columns, i, arena);
-                }
+                this->add(place, columns, i, arena);
             }
         } else {
             this->set_flag(place);
@@ -245,30 +243,30 @@ public:
 
         if (has_null) {
             for (size_t i = batch_begin; i <= batch_end; ++i) {
-                if (!column->is_null_at(i)) {
-                    this->set_flag(place);
-                    this->add(place, columns, i, arena);
-                }
+                this->add(place, columns, i, arena);
             }
         } else {
             this->set_flag(place);
             const IColumn* nested_column = &column->get_nested_column();
-            this->nested_function->add_batch_range(
-                    batch_begin, batch_end, this->nested_place(place), &nested_column, arena);
+            this->nested_function->add_batch_range(batch_begin, batch_end,
+                                                   this->nested_place(place), &nested_column, arena,
+                                                   false);
         }
     }
 };
 
-template <bool result_is_nullable>
-class AggregateFunctionNullVariadic final
-        : public AggregateFunctionNullBase<result_is_nullable,
-                                           AggregateFunctionNullVariadic<result_is_nullable>> {
+template <typename NestFuction, bool result_is_nullable>
+class AggregateFunctionNullVariadicInline final
+        : public AggregateFunctionNullBaseInline<
+                  NestFuction, result_is_nullable,
+                  AggregateFunctionNullVariadicInline<NestFuction, result_is_nullable>> {
 public:
-    AggregateFunctionNullVariadic(AggregateFunctionPtr nested_function_, const DataTypes& arguments,
-                                  const Array& params)
-            : AggregateFunctionNullBase<result_is_nullable,
-                                        AggregateFunctionNullVariadic<result_is_nullable>>(
-                      std::move(nested_function_), arguments, params),
+    AggregateFunctionNullVariadicInline(IAggregateFunction* nested_function_,
+                                        const DataTypes& arguments)
+            : AggregateFunctionNullBaseInline<
+                      NestFuction, result_is_nullable,
+                      AggregateFunctionNullVariadicInline<NestFuction, result_is_nullable>>(
+                      nested_function_, arguments),
               number_of_arguments(arguments.size()) {
         if (number_of_arguments == 1) {
             LOG(FATAL)
@@ -322,5 +320,4 @@ private:
     std::array<char, MAX_ARGS>
             is_nullable; /// Plain array is better than std::vector due to one indirection less.
 };
-
 } // namespace doris::vectorized
