@@ -379,6 +379,10 @@ public:
         LOG(FATAL) << "should not call the method in column object";
     }
 
+    void replicate(const uint32_t* indexs, size_t target_size, IColumn& column) const override {
+        LOG(FATAL) << "not support";
+    }
+
     template <typename Func>
     ColumnPtr apply_for_subcolumns(Func&& func, std::string_view func_name) const;
 
