@@ -72,10 +72,9 @@ using KeysNullMap = std::array<UInt8, get_bitmap_size<T>()>;
 /// Pack into a binary blob of type T a set of fixed-size keys. Granted that all the keys fit into the
 /// binary blob, they are disposed in it consecutively.
 template <typename T, bool has_low_cardinality = false>
-static inline T ALWAYS_INLINE
-pack_fixed(size_t i, size_t keys_size, const ColumnRawPtrs& key_columns, const Sizes& key_sizes,
-           const ColumnRawPtrs* low_cardinality_positions [[maybe_unused]] = nullptr,
-           const Sizes* low_cardinality_sizes [[maybe_unused]] = nullptr) {
+T pack_fixed(size_t i, size_t keys_size, const ColumnRawPtrs& key_columns, const Sizes& key_sizes,
+             const ColumnRawPtrs* low_cardinality_positions [[maybe_unused]] = nullptr,
+             const Sizes* low_cardinality_sizes [[maybe_unused]] = nullptr) {
     union {
         T key;
         char bytes[sizeof(key)] = {};
@@ -149,9 +148,8 @@ pack_fixed(size_t i, size_t keys_size, const ColumnRawPtrs& key_columns, const S
 
 /// Similar as above but supports nullable values.
 template <typename T>
-static inline T ALWAYS_INLINE pack_fixed(size_t i, size_t keys_size,
-                                         const ColumnRawPtrs& key_columns, const Sizes& key_sizes,
-                                         const KeysNullMap<T>& bitmap) {
+T pack_fixed(size_t i, size_t keys_size, const ColumnRawPtrs& key_columns, const Sizes& key_sizes,
+             const KeysNullMap<T>& bitmap) {
     union {
         T key;
         char bytes[sizeof(key)] = {};
@@ -168,17 +166,18 @@ static inline T ALWAYS_INLINE pack_fixed(size_t i, size_t keys_size,
     }
 
     for (size_t j = 0; j < keys_size; ++j) {
-        bool is_null;
+        bool is_null = false;
 
-        if (!has_bitmap)
-            is_null = false;
-        else {
+        if (has_bitmap) {
             size_t bucket = j / 8;
             size_t off = j % 8;
             is_null = ((bitmap[bucket] >> off) & 1) == 1;
         }
 
-        if (is_null) continue;
+        if (is_null) {
+            offset += key_sizes[j];
+            continue;
+        }
 
         switch (key_sizes[j]) {
         case 1:
@@ -186,28 +185,24 @@ static inline T ALWAYS_INLINE pack_fixed(size_t i, size_t keys_size,
                    static_cast<const ColumnVectorHelper*>(key_columns[j])->get_raw_data_begin<1>() +
                            i,
                    1);
-            offset += 1;
             break;
         case 2:
             memcpy(bytes + offset,
                    static_cast<const ColumnVectorHelper*>(key_columns[j])->get_raw_data_begin<2>() +
                            i * 2,
                    2);
-            offset += 2;
             break;
         case 4:
             memcpy(bytes + offset,
                    static_cast<const ColumnVectorHelper*>(key_columns[j])->get_raw_data_begin<4>() +
                            i * 4,
                    4);
-            offset += 4;
             break;
         case 8:
             memcpy(bytes + offset,
                    static_cast<const ColumnVectorHelper*>(key_columns[j])->get_raw_data_begin<8>() +
                            i * 8,
                    8);
-            offset += 8;
             break;
         default:
             memcpy(bytes + offset,
@@ -216,18 +211,21 @@ static inline T ALWAYS_INLINE pack_fixed(size_t i, size_t keys_size,
                    key_sizes[j]);
             offset += key_sizes[j];
         }
+
+        offset += key_sizes[j];
     }
 
     return key;
 }
 
 /// Hash a set of keys into a UInt128 value.
-static inline UInt128 ALWAYS_INLINE hash128(size_t i, size_t keys_size,
-                                            const ColumnRawPtrs& key_columns) {
+inline UInt128 hash128(size_t i, size_t keys_size, const ColumnRawPtrs& key_columns) {
     UInt128 key;
     SipHash hash;
 
-    for (size_t j = 0; j < keys_size; ++j) key_columns[j]->update_hash_with_value(i, hash);
+    for (size_t j = 0; j < keys_size; ++j) {
+        key_columns[j]->update_hash_with_value(i, hash);
+    }
 
     hash.get128(key.low, key.high);
 
@@ -235,8 +233,7 @@ static inline UInt128 ALWAYS_INLINE hash128(size_t i, size_t keys_size,
 }
 
 /// Copy keys to the pool. Then put into pool StringRefs to them and return the pointer to the first.
-static inline StringRef* ALWAYS_INLINE place_keys_in_pool(size_t keys_size, StringRefs& keys,
-                                                          Arena& pool) {
+inline StringRef* place_keys_in_pool(size_t keys_size, StringRefs& keys, Arena& pool) {
     for (size_t j = 0; j < keys_size; ++j) {
         char* place = pool.alloc(keys[j].size);
         memcpy_small_allow_read_write_overflow15(place, keys[j].data, keys[j].size);
@@ -252,13 +249,14 @@ static inline StringRef* ALWAYS_INLINE place_keys_in_pool(size_t keys_size, Stri
 
 /** Serialize keys into a continuous chunk of memory.
   */
-static inline StringRef ALWAYS_INLINE serialize_keys_to_pool_contiguous(
-        size_t i, size_t keys_size, const ColumnRawPtrs& key_columns, Arena& pool) {
+inline StringRef serialize_keys_to_pool_contiguous(size_t i, size_t keys_size,
+                                                   const ColumnRawPtrs& key_columns, Arena& pool) {
     const char* begin = nullptr;
 
     size_t sum_size = 0;
-    for (size_t j = 0; j < keys_size; ++j)
+    for (size_t j = 0; j < keys_size; ++j) {
         sum_size += key_columns[j]->serialize_value_into_arena(i, pool, begin).size;
+    }
 
     return {begin, sum_size};
 }
