@@ -857,7 +857,7 @@ Status Block::serialize(int be_exec_version, PBlock* pblock,
         *compressed_bytes = content_uncompressed_size;
     }
     if (!allow_transfer_large_data && *compressed_bytes >= std::numeric_limits<int32_t>::max()) {
-        return Status::InternalError("The block is large than 2GB({}), can not send by Protobuf.",
+        return Status::InternalError("The block is larger than 2GB({}), can not send by Protobuf.",
                                      *compressed_bytes);
     }
     return Status::OK();
