@@ -17,6 +17,8 @@
 
 #pragma once
 
+#include <gen_cpp/PlanNodes_types.h>
+
 #include "common/factory_creator.h"
 #include "common/status.h"
 #include "runtime/types.h"
@@ -30,7 +32,13 @@ class Block;
 // a set of blocks with specified schema,
 class GenericReader {
 public:
+    GenericReader() : _push_down_agg_type(TPushAggOp::type::NONE) {}
+    void set_push_down_agg_type(TPushAggOp::type push_down_agg_type) {
+        _push_down_agg_type = push_down_agg_type;
+    }
+
     virtual Status get_next_block(Block* block, size_t* read_rows, bool* eof) = 0;
+
     virtual std::unordered_map<std::string, TypeDescriptor> get_name_to_type() {
         std::unordered_map<std::string, TypeDescriptor> map;
         return map;
@@ -67,6 +75,7 @@ protected:
 
     /// Whether the underlying FileReader has filled the partition&missing columns
     bool _fill_all_columns = false;
+    TPushAggOp::type _push_down_agg_type;
 };
 
 } // namespace doris::vectorized
