@@ -43,13 +43,11 @@
 namespace doris::vectorized {
 
 ScannerContext::ScannerContext(doris::RuntimeState* state_, doris::vectorized::VScanNode* parent,
-                               const doris::TupleDescriptor* input_tuple_desc,
                                const doris::TupleDescriptor* output_tuple_desc,
                                const std::list<VScannerSPtr>& scanners_, int64_t limit_,
                                int64_t max_bytes_in_blocks_queue_, const int num_parallel_instances)
         : _state(state_),
           _parent(parent),
-          _input_tuple_desc(input_tuple_desc),
           _output_tuple_desc(output_tuple_desc),
           _process_status(Status::OK()),
           _batch_size(state_->batch_size()),
@@ -69,7 +67,6 @@ ScannerContext::ScannerContext(doris::RuntimeState* state_, doris::vectorized::V
 
 // After init function call, should not access _parent
 Status ScannerContext::init() {
-    _real_tuple_desc = _input_tuple_desc != nullptr ? _input_tuple_desc : _output_tuple_desc;
     // 1. Calculate max concurrency
     // TODO: now the max thread num <= config::doris_scanner_thread_pool_thread_num / 4
     // should find a more reasonable value.
@@ -107,9 +104,6 @@ Status ScannerContext::init() {
             limit == -1 ? _batch_size : std::min(static_cast<int64_t>(_batch_size), limit);
     _block_per_scanner = (doris_scanner_row_num + (real_block_size - 1)) / real_block_size;
     _free_blocks_capacity = _max_thread_num * _block_per_scanner;
-    auto pre_alloc_block_count = _max_thread_num * _block_per_scanner;
-
-    _init_free_block(pre_alloc_block_count, real_block_size);
 
 #ifndef BE_TEST
     // 3. get thread token
@@ -129,19 +123,6 @@ Status ScannerContext::init() {
     return Status::OK();
 }
 
-void ScannerContext::_init_free_block(int pre_alloc_block_count, int real_block_size) {
-    // The free blocks is used for final output block of scanners.
-    // So use _output_tuple_desc;
-    int64_t free_blocks_memory_usage = 0;
-    for (int i = 0; i < pre_alloc_block_count; ++i) {
-        auto block = vectorized::Block::create_unique(_output_tuple_desc->slots(), real_block_size,
-                                                      true /*ignore invalid slots*/);
-        free_blocks_memory_usage += block->allocated_bytes();
-        _free_blocks.enqueue(std::move(block));
-    }
-    _free_blocks_memory_usage->add(free_blocks_memory_usage);
-}
-
 vectorized::BlockUPtr ScannerContext::get_free_block(bool* has_free_block,
                                                      bool get_block_not_empty) {
     vectorized::BlockUPtr block;
@@ -154,7 +135,7 @@ vectorized::BlockUPtr ScannerContext::get_free_block(bool* has_free_block,
     }
 
     COUNTER_UPDATE(_newly_create_free_blocks_num, 1);
-    return vectorized::Block::create_unique(_real_tuple_desc->slots(), _batch_size,
+    return vectorized::Block::create_unique(_output_tuple_desc->slots(), _batch_size,
                                             true /*ignore invalid slots*/);
 }
 
