@@ -17,15 +17,13 @@
 
 #include "vec/exec/vmysql_scan_node.h"
 
+#include <gen_cpp/PlanNodes_types.h>
+
 #include "exec/text_converter.h"
-#include "exec/text_converter.hpp"
-#include "gen_cpp/PlanNodes_types.h"
-#include "runtime/row_batch.h"
 #include "runtime/runtime_state.h"
-#include "runtime/string_value.h"
-#include "runtime/tuple_row.h"
 #include "util/runtime_profile.h"
 #include "util/types.h"
+#include "vec/common/string_ref.h"
 namespace doris::vectorized {
 
 VMysqlScanNode::VMysqlScanNode(ObjectPool* pool, const TPlanNode& tnode, const DescriptorTbl& descs)
@@ -75,19 +73,13 @@ Status VMysqlScanNode::prepare(RuntimeState* state) {
     // new one scanner
     _mysql_scanner.reset(new (std::nothrow) MysqlScanner(_my_param));
 
-    if (_mysql_scanner.get() == nullptr) {
+    if (_mysql_scanner == nullptr) {
         return Status::InternalError("new a mysql scanner failed.");
     }
 
-    _tuple_pool.reset(new (std::nothrow) MemPool());
-
-    if (_tuple_pool.get() == nullptr) {
-        return Status::InternalError("new a mem pool failed.");
-    }
-
     _text_converter.reset(new (std::nothrow) TextConverter('\\'));
 
-    if (_text_converter.get() == nullptr) {
+    if (_text_converter == nullptr) {
         return Status::InternalError("new a text convertor failed.");
     }
 
@@ -100,7 +92,6 @@ Status VMysqlScanNode::open(RuntimeState* state) {
     if (nullptr == state) {
         return Status::InternalError("input pointer is nullptr.");
     }
-    START_AND_SCOPE_SPAN(state->get_tracer(), span, "VMysqlScanNode::open");
     SCOPED_TIMER(_runtime_profile->total_time_counter());
     RETURN_IF_ERROR(ExecNode::open(state));
     VLOG_CRITICAL << "MysqlScanNode::Open";
@@ -129,24 +120,10 @@ Status VMysqlScanNode::open(RuntimeState* state) {
     return Status::OK();
 }
 
-Status VMysqlScanNode::write_text_slot(char* value, int value_length, SlotDescriptor* slot,
-                                       RuntimeState* state) {
-    if (!_text_converter->write_slot(slot, _tuple, value, value_length, true, false,
-                                     _tuple_pool.get())) {
-        std::stringstream ss;
-        ss << "Fail to convert mysql value:'" << value << "' to " << slot->type() << " on column:`"
-           << slot->col_name() + "`";
-        return Status::InternalError(ss.str());
-    }
-
-    return Status::OK();
-}
-
 Status VMysqlScanNode::get_next(RuntimeState* state, vectorized::Block* block, bool* eos) {
     if (state == nullptr || block == nullptr || eos == nullptr) {
         return Status::InternalError("input is nullptr");
     }
-    INIT_AND_SCOPE_GET_NEXT_SPAN(state->get_tracer(), _get_next_span, "VMysqlScanNode::get_next");
     VLOG_CRITICAL << "VMysqlScanNode::GetNext";
 
     if (!_is_init) {
@@ -239,11 +216,8 @@ Status VMysqlScanNode::close(RuntimeState* state) {
     if (is_closed()) {
         return Status::OK();
     }
-    START_AND_SCOPE_SPAN(state->get_tracer(), span, "VMysqlScanNode::close");
     SCOPED_TIMER(_runtime_profile->total_time_counter());
 
-    _tuple_pool.reset();
-
     return ExecNode::close(state);
 }
 
