@@ -17,41 +17,44 @@
 
 #pragma once
 
+#include <iosfwd>
 #include <memory>
+#include <string>
+#include <vector>
 
+#include "common/global_types.h"
 #include "exec/scan_node.h"
 #include "exec/schema_scanner.h"
-#include "gen_cpp/Descriptors_types.h"
-#include "runtime/descriptors.h"
+
 namespace doris {
 
-class TextConverter;
 class TupleDescriptor;
 class RuntimeState;
 class Status;
+class DescriptorTbl;
+class ObjectPool;
+class TPlanNode;
+class TScanRangeParams;
 
 namespace vectorized {
+class Block;
 
 class VSchemaScanNode : public ScanNode {
 public:
     VSchemaScanNode(ObjectPool* pool, const TPlanNode& tnode, const DescriptorTbl& descs);
-    ~VSchemaScanNode();
+    ~VSchemaScanNode() override;
     Status prepare(RuntimeState* state) override;
-    Status get_next(RuntimeState* state, RowBatch* row_batch, bool* eos) override {
-        return Status::NotSupported("Not Implemented VSchemaScanNode Node::get_next scalar");
-    }
-    virtual Status get_next(RuntimeState* state, vectorized::Block* block, bool* eos) override;
+    Status get_next(RuntimeState* state, vectorized::Block* block, bool* eos) override;
 
     // Prepare conjuncts, create Schema columns to slots mapping
     // initialize schema_scanner
     Status init(const TPlanNode& tnode, RuntimeState* state = nullptr) override;
     // Start Schema scan using schema_scanner.
     Status open(RuntimeState* state) override;
-
-private:
     // Close the schema_scanner, and report errors.
     Status close(RuntimeState* state) override;
 
+private:
     // this is no use in this class
     Status set_scan_ranges(const std::vector<TScanRangeParams>& scan_ranges) override;
 
@@ -64,28 +67,14 @@ private:
     // Tuple id resolved in prepare() to set _tuple_desc;
     TupleId _tuple_id;
 
-    // Descriptor of tuples read from schema table.
-    const TupleDescriptor* _src_tuple_desc;
     // Descriptor of dest tuples
     const TupleDescriptor* _dest_tuple_desc;
     // Tuple index in tuple row.
     int _tuple_idx;
     // slot num need to fill in and return
     int _slot_num;
-    // Pool for allocating tuple data, including all varying-length slots.
-    std::unique_ptr<MemPool> _tuple_pool;
     // Jni helper for scanning an schema table.
     std::unique_ptr<SchemaScanner> _schema_scanner;
-    // Current tuple.
-    doris::Tuple* _src_tuple;
-    // Map from index in slots to column of schema table.
-    std::vector<int> _index_map;
-
-    Status write_slot_to_vectorized_column(void* slot, SlotDescriptor* slot_desc,
-                                           vectorized::MutableColumnPtr* col_ptr);
-    void project_tuple();
-    doris::Tuple* _src_single_tuple;
-    doris::Tuple* _dest_single_tuple;
 };
 } // namespace vectorized
 } // namespace doris
\ No newline at end of file
