@@ -17,23 +17,44 @@
 
 #pragma once
 
-#include "exprs/table_function/explode_bitmap.h"
-#include "exprs/table_function/table_function.h"
+#include <stddef.h>
+
+#include <memory>
+
+#include "common/status.h"
 #include "util/bitmap_value.h"
-#include "vec/columns/column.h"
+#include "vec/data_types/data_type.h"
+#include "vec/exprs/table_function/table_function.h"
+
+namespace doris {
+namespace vectorized {
+class Block;
+} // namespace vectorized
+} // namespace doris
 
 namespace doris::vectorized {
 
-class VExplodeBitmapTableFunction : public ExplodeBitmapTableFunction {
+class VExplodeBitmapTableFunction final : public TableFunction {
+    ENABLE_FACTORY_CREATOR(VExplodeBitmapTableFunction);
+
 public:
     VExplodeBitmapTableFunction();
+    ~VExplodeBitmapTableFunction() override = default;
 
-    Status process_init(vectorized::Block* block) override;
+    Status reset() override;
+    void get_value(MutableColumnPtr& column) override;
+    Status forward(int step = 1) override;
+
+    Status process_init(Block* block) override;
     Status process_row(size_t row_idx) override;
     Status process_close() override;
-    Status get_value_length(int64_t* length) override;
 
 private:
+    void _reset_iterator();
+    // Not own object, just a reference
+    const BitmapValue* _cur_bitmap = nullptr;
+    // iterator of _cur_bitmap
+    std::unique_ptr<BitmapValueIterator> _cur_iter = nullptr;
     ColumnPtr _value_column;
 };
 
