@@ -17,12 +17,34 @@
 
 #pragma once
 
+#include <stdint.h>
+
+#include <atomic>
+#include <string>
+#include <vector>
+
+#include "common/config.h"
+#include "common/object_pool.h"
+#include "common/status.h"
+#include "udf/udf.h"
 #include "vec/exprs/vexpr.h"
 
+namespace doris {
+class RowDescriptor;
+class RuntimeState;
+class TExprNode;
+namespace vectorized {
+class Block;
+class VExprContext;
+} // namespace vectorized
+} // namespace doris
+
 namespace doris::vectorized {
 class VRuntimeFilterWrapper final : public VExpr {
+    ENABLE_FACTORY_CREATOR(VRuntimeFilterWrapper);
+
 public:
-    VRuntimeFilterWrapper(const TExprNode& node, VExpr* impl);
+    VRuntimeFilterWrapper(const TExprNode& node, const VExprSPtr& impl);
     VRuntimeFilterWrapper(const VRuntimeFilterWrapper& vexpr);
     ~VRuntimeFilterWrapper() override = default;
     doris::Status execute(VExprContext* context, doris::vectorized::Block* block,
@@ -31,16 +53,14 @@ public:
                           VExprContext* context) override;
     doris::Status open(doris::RuntimeState* state, VExprContext* context,
                        FunctionContext::FunctionStateScope scope) override;
-    std::string debug_string() const override { return _impl->debug_string(); };
+    std::string debug_string() const override { return _impl->debug_string(); }
     bool is_constant() const override;
-    void close(doris::RuntimeState* state, VExprContext* context,
-               FunctionContext::FunctionStateScope scope) override;
-    VExpr* clone(doris::ObjectPool* pool) const override {
-        return pool->add(new VRuntimeFilterWrapper(*this));
-    }
+    void close(VExprContext* context, FunctionContext::FunctionStateScope scope) override;
+    VExprSPtr clone() const override { return VRuntimeFilterWrapper::create_shared(*this); }
     const std::string& expr_name() const override;
+    const VExprSPtrs& children() const override { return _impl->children(); }
 
-    const VExpr* get_impl() const override { return _impl; }
+    const VExprSPtr get_impl() const override { return _impl; }
 
     // if filter rate less than this, bloom filter will set always true
     constexpr static double EXPECTED_FILTER_RATE = 0.4;
@@ -57,7 +77,7 @@ public:
     }
 
 private:
-    VExpr* _impl;
+    VExprSPtr _impl;
 
     bool _always_true;
     /// TODO: statistic filter rate in the profile
