@@ -35,6 +35,7 @@
 #include "vec/columns/column_map.h"
 #include "vec/columns/column_nullable.h"
 #include "vec/columns/column_string.h"
+#include "vec/columns/column_struct.h"
 #include "vec/columns/column_vector.h"
 #include "vec/columns/columns_number.h"
 #include "vec/core/block.h"
@@ -283,6 +284,44 @@ private:
         return _execute_nullable(args, input_rows_count, src_null_map, dst_null_map);
     }
 
+    ColumnPtr _execute_common(const ColumnArray::Offsets64& offsets, const IColumn& nested_column,
+                              const UInt8* arr_null_map, const IColumn& indices,
+                              const UInt8* nested_null_map, UInt8* dst_null_map) {
+        // prepare return data
+        auto dst_column = nested_column.clone_empty();
+        dst_column->reserve(offsets.size());
+
+        // process
+        for (size_t row = 0; row < offsets.size(); ++row) {
+            size_t off = offsets[row - 1];
+            size_t len = offsets[row] - off;
+            auto index = indices.get_int(row);
+            // array is nullable
+            bool null_flag = bool(arr_null_map && arr_null_map[row]);
+            // calc index in nested column
+            if (!null_flag && index > 0 && index <= len) {
+                index += off - 1;
+            } else if (!null_flag && index < 0 && -index <= len) {
+                index += off + len;
+            } else {
+                null_flag = true;
+            }
+            // nested column nullable check
+            if (!null_flag && nested_null_map && nested_null_map[index]) {
+                null_flag = true;
+            }
+            // actual data copy
+            if (!null_flag) {
+                dst_null_map[row] = false;
+                dst_column->insert_from(nested_column, index);
+            } else {
+                dst_null_map[row] = true;
+                dst_column->insert_default();
+            }
+        }
+        return dst_column;
+    }
+
     ColumnPtr _execute_nullable(const ColumnsWithTypeAndName& arguments, size_t input_rows_count,
                                 const UInt8* src_null_map, UInt8* dst_null_map) {
         // check array nested column type and get data
@@ -356,6 +395,9 @@ private:
         } else if (check_column<ColumnString>(*nested_column)) {
             res = _execute_string(offsets, *nested_column, src_null_map, *idx_col, nested_null_map,
                                   dst_null_map);
+        } else {
+            res = _execute_common(offsets, *nested_column, src_null_map, *idx_col, nested_null_map,
+                                  dst_null_map);
         }
 
         return res;
