@@ -34,6 +34,7 @@
 #include "util/hash_util.hpp"
 #include "util/time_lut.h"
 #include "util/timezone_utils.h"
+#include "vec/common/hash_table/phmap_fwd_decl.h"
 
 namespace cctz {
 class time_zone;
@@ -43,7 +44,7 @@ namespace doris {
 
 namespace vectorized {
 
-using ZoneList = std::map<std::string, cctz::time_zone>;
+using ZoneList = std::unordered_map<std::string, cctz::time_zone>;
 
 enum TimeUnit {
     MICROSECOND,
@@ -354,7 +355,7 @@ public:
     // 'YYYYMMDDTHHMMSS'
     bool from_date_str(const char* str, int len);
     bool from_date_str(const char* str, int len, const cctz::time_zone& local_time_zone,
-                       ZoneList& time_zone_cache);
+                       ZoneList& time_zone_cache, std::shared_mutex* cache_lock);
 
     // Construct Date/Datetime type value from int64_t value.
     // Return true if convert success. Otherwise return false.
@@ -481,6 +482,10 @@ public:
     template <TimeUnit unit>
     bool date_add_interval(const TimeInterval& interval);
 
+    // set interval
+    template <TimeUnit unit>
+    bool date_set_interval(const TimeInterval& interval);
+
     template <TimeUnit unit>
     bool datetime_trunc(); //datetime trunc, like trunc minute = 0
 
@@ -661,6 +666,9 @@ public:
 
     bool get_date_from_daynr(uint64_t);
 
+    // reset 0
+    void reset_zero_by_type(int type) { set_zero(type); }
+
 private:
     // Used to make sure sizeof VecDateTimeValue
     friend class UnusedClass;
@@ -686,7 +694,7 @@ private:
     char* to_time_buffer(char* to) const;
 
     bool from_date_str_base(const char* date_str, int len, const cctz::time_zone* local_time_zone,
-                            ZoneList* time_zone_cache);
+                            ZoneList* time_zone_cache, std::shared_mutex* cache_lock);
 
     int64_t to_date_int64() const;
     int64_t to_time_int64() const;
@@ -808,7 +816,7 @@ public:
     // 'YYYYMMDDTHHMMSS'
     bool from_date_str(const char* str, int len, int scale = -1);
     bool from_date_str(const char* str, int len, const cctz::time_zone& local_time_zone,
-                       ZoneList& time_zone_cache, int scale = -1);
+                       ZoneList& time_zone_cache, std::shared_mutex* cache_lock, int scale = -1);
 
     // Convert this value to string
     // this will check type to decide which format to convert
@@ -878,6 +886,10 @@ public:
         return hour() * SECOND_PER_HOUR + minute() * SECOND_PER_MINUTE + second();
     }
 
+    int64_t time_part_to_microsecond() const {
+        return time_part_to_seconds() * 1000 * 1000 + microsecond();
+    }
+
     uint16_t year() const { return date_v2_value_.year_; }
     uint8_t month() const { return date_v2_value_.month_; }
     int quarter() const { return (date_v2_value_.month_ - 1) / 3 + 1; }
@@ -931,6 +943,9 @@ public:
     template <TimeUnit unit>
     bool date_add_interval(const TimeInterval& interval);
 
+    template <TimeUnit unit>
+    bool date_set_interval(const TimeInterval& interval);
+
     template <TimeUnit unit>
     bool datetime_trunc(); //datetime trunc, like trunc minute = 0
 
@@ -1073,11 +1088,27 @@ public:
         return time_part_to_seconds() - rhs.time_part_to_seconds();
     }
 
+    //only calculate the diff of dd:mm:ss.SSSSSS
+    template <typename RHS>
+    int64_t time_part_diff_microsecond(const RHS& rhs) const {
+        return time_part_to_microsecond() - rhs.time_part_to_microsecond();
+    }
+
     template <typename RHS>
     int64_t second_diff(const RHS& rhs) const {
         return (daynr() - rhs.daynr()) * SECOND_PER_HOUR * HOUR_PER_DAY + time_part_diff(rhs);
     }
 
+    template <typename RHS>
+    double microsecond_diff(const RHS& rhs) const {
+        int64_t diff_m = (daynr() - rhs.daynr()) * SECOND_PER_HOUR * HOUR_PER_DAY * 1000 * 1000 +
+                         time_part_diff_microsecond(rhs);
+        if (diff_m > (int64_t)3020399 * 1000 * 1000) {
+            diff_m = (int64_t)3020399 * 1000 * 1000;
+        }
+        return diff_m;
+    }
+
     bool can_cast_to_date_without_loss_accuracy() {
         return this->hour() == 0 && this->minute() == 0 && this->second() == 0 &&
                this->microsecond() == 0;
@@ -1141,13 +1172,16 @@ public:
 
     void format_datetime(uint32_t* date_v, bool* carry_bits) const;
 
+    void set_int_val(uint64_t val) { this->int_val_ = val; }
+
 private:
     static uint8_t calc_week(const uint32_t& day_nr, const uint16_t& year, const uint8_t& month,
                              const uint8_t& day, uint8_t mode, uint16_t* to_year,
                              bool disable_lut = false);
 
     bool from_date_str_base(const char* date_str, int len, int scale,
-                            const cctz::time_zone* local_time_zone, ZoneList* time_zone_cache);
+                            const cctz::time_zone* local_time_zone, ZoneList* time_zone_cache,
+                            std::shared_mutex* cache_lock);
 
     // Used to construct from int value
     int64_t standardize_timevalue(int64_t value);
