@@ -15,10 +15,19 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#include <gtest/gtest.h>
+#include <stdint.h>
 
+#include <iomanip>
+#include <string>
+#include <vector>
+
+#include "common/status.h"
 #include "function_test_util.h"
+#include "gtest/gtest_pred_impl.h"
+#include "testutil/any_type.h"
+#include "vec/core/types.h"
 #include "vec/data_types/data_type_jsonb.h"
+#include "vec/data_types/data_type_nullable.h"
 #include "vec/data_types/data_type_number.h"
 #include "vec/data_types/data_type_string.h"
 
@@ -26,7 +35,7 @@ namespace doris::vectorized {
 using namespace ut_type;
 
 TEST(FunctionJsonbTEST, JsonbParseTest) {
-    std::string func_name = "jsonb_parse";
+    std::string func_name = "json_parse";
     InputTypeSet input_types = {Nullable {TypeIndex::String}};
 
     DataSet data_set_valid = {
@@ -85,7 +94,7 @@ TEST(FunctionJsonbTEST, JsonbParseTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseErrorToNullTest) {
-    std::string func_name = "jsonb_parse_error_to_null";
+    std::string func_name = "json_parse_error_to_null";
     InputTypeSet input_types = {Nullable {TypeIndex::String}};
 
     DataSet data_set = {
@@ -120,7 +129,7 @@ TEST(FunctionJsonbTEST, JsonbParseErrorToNullTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseErrorToValueTest) {
-    std::string func_name = "jsonb_parse_error_to_value";
+    std::string func_name = "json_parse_error_to_value";
     InputTypeSet input_types = {Nullable {TypeIndex::String}, TypeIndex::String};
 
     DataSet data_set = {
@@ -158,7 +167,7 @@ TEST(FunctionJsonbTEST, JsonbParseErrorToValueTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseErrorToInvalidTest) {
-    std::string func_name = "jsonb_parse_error_to_invalid";
+    std::string func_name = "json_parse_error_to_invalid";
     InputTypeSet input_types = {Nullable {TypeIndex::String}};
 
     DataSet data_set = {
@@ -193,7 +202,7 @@ TEST(FunctionJsonbTEST, JsonbParseErrorToInvalidTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNullableTest) {
-    std::string func_name = "jsonb_parse_nullable";
+    std::string func_name = "json_parse_nullable";
     InputTypeSet input_types = {TypeIndex::String};
 
     DataSet data_set_valid = {
@@ -252,7 +261,7 @@ TEST(FunctionJsonbTEST, JsonbParseNullableTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNullableErrorToNullTest) {
-    std::string func_name = "jsonb_parse_nullable_error_to_null";
+    std::string func_name = "json_parse_nullable_error_to_null";
     InputTypeSet input_types = {TypeIndex::String};
 
     DataSet data_set = {
@@ -287,7 +296,7 @@ TEST(FunctionJsonbTEST, JsonbParseNullableErrorToNullTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNullableErrorToValueTest) {
-    std::string func_name = "jsonb_parse_nullable_error_to_value";
+    std::string func_name = "json_parse_nullable_error_to_value";
     InputTypeSet input_types = {TypeIndex::String, TypeIndex::String};
 
     DataSet data_set = {
@@ -325,7 +334,7 @@ TEST(FunctionJsonbTEST, JsonbParseNullableErrorToValueTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNullableErrorToInvalidTest) {
-    std::string func_name = "jsonb_parse_nullable_error_to_invalid";
+    std::string func_name = "json_parse_nullable_error_to_invalid";
     InputTypeSet input_types = {TypeIndex::String};
 
     DataSet data_set = {
@@ -360,7 +369,7 @@ TEST(FunctionJsonbTEST, JsonbParseNullableErrorToInvalidTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNotnullTest) {
-    std::string func_name = "jsonb_parse_notnull";
+    std::string func_name = "json_parse_notnull";
     InputTypeSet input_types = {TypeIndex::String};
 
     DataSet data_set_valid = {
@@ -418,7 +427,7 @@ TEST(FunctionJsonbTEST, JsonbParseNotnullTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNotnullErrorToValueTest) {
-    std::string func_name = "jsonb_parse_notnull_error_to_value";
+    std::string func_name = "json_parse_notnull_error_to_value";
     InputTypeSet input_types = {TypeIndex::String, TypeIndex::String};
 
     DataSet data_set = {
@@ -455,7 +464,7 @@ TEST(FunctionJsonbTEST, JsonbParseNotnullErrorToValueTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbParseNotnullErrorToInvalidTest) {
-    std::string func_name = "jsonb_parse_notnull_error_to_invalid";
+    std::string func_name = "json_parse_notnull_error_to_invalid";
     InputTypeSet input_types = {TypeIndex::String};
 
     DataSet data_set = {
@@ -492,7 +501,7 @@ TEST(FunctionJsonbTEST, JsonbExtractTest) {
     std::string func_name = "jsonb_extract";
     InputTypeSet input_types = {TypeIndex::JSONB, TypeIndex::String};
 
-    // jsonb_extract root
+    // json_extract root
     DataSet data_set = {
             {{Null(), STRING("$")}, Null()},
             {{STRING("null"), STRING("$")}, STRING("null")},
@@ -519,7 +528,7 @@ TEST(FunctionJsonbTEST, JsonbExtractTest) {
 
     check_function<DataTypeJsonb, true>(func_name, input_types, data_set);
 
-    // jsonb_extract obejct
+    // json_extract obejct
     data_set = {
             {{Null(), STRING("$.k1")}, Null()},
             {{STRING("null"), STRING("$.k1")}, Null()},
@@ -544,21 +553,22 @@ TEST(FunctionJsonbTEST, JsonbExtractTest) {
 
     check_function<DataTypeJsonb, true>(func_name, input_types, data_set);
 
-    // jsonb_extract array
+    // json_extract array
     data_set = {
             {{Null(), STRING("$[0]")}, Null()},
             {{STRING("null"), STRING("$[0]")}, Null()},
             {{STRING("true"), STRING("$[0]")}, Null()},
             {{STRING("false"), STRING("$[0]")}, Null()},
-            {{STRING("100"), STRING("$[0]")}, Null()},                        //int8
-            {{STRING("10000"), STRING("$[0]")}, Null()},                      // int16
-            {{STRING("1000000000"), STRING("$[0]")}, Null()},                 // int32
-            {{STRING("1152921504606846976"), STRING("$[0]")}, Null()},        // int64
-            {{STRING("6.18"), STRING("$[0]")}, Null()},                       // double
-            {{STRING(R"("abcd")"), STRING("$[0]")}, Null()},                  // string
-            {{STRING("{}"), STRING("$[0]")}, Null()},                         // empty object
-            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0]")}, Null()}, // object
-            {{STRING("[]"), STRING("$[0]")}, Null()},                         // empty array
+            {{STRING("100"), STRING("$[0]")}, Null()},                 //int8
+            {{STRING("10000"), STRING("$[0]")}, Null()},               // int16
+            {{STRING("1000000000"), STRING("$[0]")}, Null()},          // int32
+            {{STRING("1152921504606846976"), STRING("$[0]")}, Null()}, // int64
+            {{STRING("6.18"), STRING("$[0]")}, Null()},                // double
+            {{STRING(R"("abcd")"), STRING("$[0]")}, Null()},           // string
+            {{STRING("{}"), STRING("$[0]")}, STRING("{}")},            // empty object
+            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0]")},
+             STRING(R"({"k1":"v31","k2":300})")},     // object
+            {{STRING("[]"), STRING("$[0]")}, Null()}, // empty array
             {{STRING("null"), STRING("$[1]")}, Null()},
             {{STRING("true"), STRING("$[1]")}, Null()},
             {{STRING("false"), STRING("$[1]")}, Null()},
@@ -605,23 +615,24 @@ TEST(FunctionJsonbTEST, JsonbExtractTest) {
 
     check_function<DataTypeJsonb, true>(func_name, input_types, data_set);
 
-    // jsonb_extract $[0].k1
+    // json_extract $[0].k1
     data_set = {
             {{Null(), STRING("$[0].k1")}, Null()},
             {{STRING("null"), STRING("$[0].k1")}, Null()},
             {{STRING("true"), STRING("$[0].k1")}, Null()},
             {{STRING("false"), STRING("$[0].k1")}, Null()},
-            {{STRING("100"), STRING("$[0].k1")}, Null()},                        //int8
-            {{STRING("10000"), STRING("$[0].k1")}, Null()},                      // int16
-            {{STRING("1000000000"), STRING("$[0].k1")}, Null()},                 // int32
-            {{STRING("1152921504606846976"), STRING("$[0].k1")}, Null()},        // int64
-            {{STRING("6.18"), STRING("$[0].k1")}, Null()},                       // double
-            {{STRING(R"("abcd")"), STRING("$[0].k1")}, Null()},                  // string
-            {{STRING("{}"), STRING("$[0].k1")}, Null()},                         // empty object
-            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0].k1")}, Null()}, // object
-            {{STRING("[]"), STRING("$[0].k1")}, Null()},                         // empty array
-            {{STRING("[123, 456]"), STRING("$[0].k1")}, Null()},                 // int array
-            {{STRING(R"(["abc", "def"])"), STRING("$[0].k1")}, Null()},          // string array
+            {{STRING("100"), STRING("$[0].k1")}, Null()},                 //int8
+            {{STRING("10000"), STRING("$[0].k1")}, Null()},               // int16
+            {{STRING("1000000000"), STRING("$[0].k1")}, Null()},          // int32
+            {{STRING("1152921504606846976"), STRING("$[0].k1")}, Null()}, // int64
+            {{STRING("6.18"), STRING("$[0].k1")}, Null()},                // double
+            {{STRING(R"("abcd")"), STRING("$[0].k1")}, Null()},           // string
+            {{STRING("{}"), STRING("$[0].k1")}, Null()},                  // empty object
+            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0].k1")},
+             STRING(R"("v31")")},                                       // object
+            {{STRING("[]"), STRING("$[0].k1")}, Null()},                // empty array
+            {{STRING("[123, 456]"), STRING("$[0].k1")}, Null()},        // int array
+            {{STRING(R"(["abc", "def"])"), STRING("$[0].k1")}, Null()}, // string array
             {{STRING(R"([null, true, false, 100, 6.18, "abc"])"), STRING("$[0].k1")},
              Null()}, // multi type array
             {{STRING(R"([{"k1":"v41", "k2": 400}, 1, "a", 3.14])"), STRING("$[0].k1")},
@@ -632,10 +643,10 @@ TEST(FunctionJsonbTEST, JsonbExtractTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbExtractStringTest) {
-    std::string func_name = "jsonb_extract_string";
+    std::string func_name = "json_extract_string";
     InputTypeSet input_types = {TypeIndex::JSONB, TypeIndex::String};
 
-    // jsonb_extract root
+    // json_extract root
     DataSet data_set = {
             {{Null(), STRING("$")}, Null()},
             {{STRING("null"), STRING("$")}, STRING("null")},
@@ -662,7 +673,7 @@ TEST(FunctionJsonbTEST, JsonbExtractStringTest) {
 
     check_function<DataTypeString, true>(func_name, input_types, data_set);
 
-    // jsonb_extract obejct
+    // json_extract obejct
     data_set = {
             {{Null(), STRING("$.k1")}, Null()},
             {{STRING("null"), STRING("$.k1")}, Null()},
@@ -687,21 +698,22 @@ TEST(FunctionJsonbTEST, JsonbExtractStringTest) {
 
     check_function<DataTypeString, true>(func_name, input_types, data_set);
 
-    // jsonb_extract array
+    // json_extract array
     data_set = {
             {{Null(), STRING("$[0]")}, Null()},
             {{STRING("null"), STRING("$[0]")}, Null()},
             {{STRING("true"), STRING("$[0]")}, Null()},
             {{STRING("false"), STRING("$[0]")}, Null()},
-            {{STRING("100"), STRING("$[0]")}, Null()},                        //int8
-            {{STRING("10000"), STRING("$[0]")}, Null()},                      // int16
-            {{STRING("1000000000"), STRING("$[0]")}, Null()},                 // int32
-            {{STRING("1152921504606846976"), STRING("$[0]")}, Null()},        // int64
-            {{STRING("6.18"), STRING("$[0]")}, Null()},                       // double
-            {{STRING(R"("abcd")"), STRING("$[0]")}, Null()},                  // string
-            {{STRING("{}"), STRING("$[0]")}, Null()},                         // empty object
-            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0]")}, Null()}, // object
-            {{STRING("[]"), STRING("$[0]")}, Null()},                         // empty array
+            {{STRING("100"), STRING("$[0]")}, Null()},                 //int8
+            {{STRING("10000"), STRING("$[0]")}, Null()},               // int16
+            {{STRING("1000000000"), STRING("$[0]")}, Null()},          // int32
+            {{STRING("1152921504606846976"), STRING("$[0]")}, Null()}, // int64
+            {{STRING("6.18"), STRING("$[0]")}, Null()},                // double
+            {{STRING(R"("abcd")"), STRING("$[0]")}, Null()},           // string
+            {{STRING("{}"), STRING("$[0]")}, STRING("{}")},            // empty object
+            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0]")},
+             STRING(R"({"k1":"v31","k2":300})")},     // object
+            {{STRING("[]"), STRING("$[0]")}, Null()}, // empty array
             {{STRING("null"), STRING("$[1]")}, Null()},
             {{STRING("true"), STRING("$[1]")}, Null()},
             {{STRING("false"), STRING("$[1]")}, Null()},
@@ -748,23 +760,23 @@ TEST(FunctionJsonbTEST, JsonbExtractStringTest) {
 
     check_function<DataTypeString, true>(func_name, input_types, data_set);
 
-    // jsonb_extract $[0].k1
+    // json_extract $[0].k1
     data_set = {
             {{Null(), STRING("$[0].k1")}, Null()},
             {{STRING("null"), STRING("$[0].k1")}, Null()},
             {{STRING("true"), STRING("$[0].k1")}, Null()},
             {{STRING("false"), STRING("$[0].k1")}, Null()},
-            {{STRING("100"), STRING("$[0].k1")}, Null()},                        //int8
-            {{STRING("10000"), STRING("$[0].k1")}, Null()},                      // int16
-            {{STRING("1000000000"), STRING("$[0].k1")}, Null()},                 // int32
-            {{STRING("1152921504606846976"), STRING("$[0].k1")}, Null()},        // int64
-            {{STRING("6.18"), STRING("$[0].k1")}, Null()},                       // double
-            {{STRING(R"("abcd")"), STRING("$[0].k1")}, Null()},                  // string
-            {{STRING("{}"), STRING("$[0].k1")}, Null()},                         // empty object
-            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0].k1")}, Null()}, // object
-            {{STRING("[]"), STRING("$[0].k1")}, Null()},                         // empty array
-            {{STRING("[123, 456]"), STRING("$[0].k1")}, Null()},                 // int array
-            {{STRING(R"(["abc", "def"])"), STRING("$[0].k1")}, Null()},          // string array
+            {{STRING("100"), STRING("$[0].k1")}, Null()},                 //int8
+            {{STRING("10000"), STRING("$[0].k1")}, Null()},               // int16
+            {{STRING("1000000000"), STRING("$[0].k1")}, Null()},          // int32
+            {{STRING("1152921504606846976"), STRING("$[0].k1")}, Null()}, // int64
+            {{STRING("6.18"), STRING("$[0].k1")}, Null()},                // double
+            {{STRING(R"("abcd")"), STRING("$[0].k1")}, Null()},           // string
+            {{STRING("{}"), STRING("$[0].k1")}, Null()},                  // empty object
+            {{STRING(R"({"k1":"v31", "k2": 300})"), STRING("$[0].k1")}, STRING(R"(v31)")}, // object
+            {{STRING("[]"), STRING("$[0].k1")}, Null()},                // empty array
+            {{STRING("[123, 456]"), STRING("$[0].k1")}, Null()},        // int array
+            {{STRING(R"(["abc", "def"])"), STRING("$[0].k1")}, Null()}, // string array
             {{STRING(R"([null, true, false, 100, 6.18, "abc"])"), STRING("$[0].k1")},
              Null()}, // multi type array
             {{STRING(R"([{"k1":"v41", "k2": 400}, 1, "a", 3.14])"), STRING("$[0].k1")},
@@ -775,10 +787,10 @@ TEST(FunctionJsonbTEST, JsonbExtractStringTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbExtractIntTest) {
-    std::string func_name = "jsonb_extract_int";
+    std::string func_name = "json_extract_int";
     InputTypeSet input_types = {TypeIndex::JSONB, TypeIndex::String};
 
-    // jsonb_extract root
+    // json_extract root
     DataSet data_set = {
             {{Null(), STRING("$")}, Null()},
             {{STRING("null"), STRING("$")}, Null()},
@@ -803,7 +815,7 @@ TEST(FunctionJsonbTEST, JsonbExtractIntTest) {
 
     check_function<DataTypeInt32, true>(func_name, input_types, data_set);
 
-    // jsonb_extract obejct
+    // json_extract obejct
     data_set = {
             {{Null(), STRING("$.k1")}, Null()},
             {{STRING("null"), STRING("$.k1")}, Null()},
@@ -828,7 +840,7 @@ TEST(FunctionJsonbTEST, JsonbExtractIntTest) {
 
     check_function<DataTypeInt32, true>(func_name, input_types, data_set);
 
-    // jsonb_extract array
+    // json_extract array
     data_set = {
             {{Null(), STRING("$[0]")}, Null()},
             {{STRING("null"), STRING("$[0]")}, Null()},
@@ -889,7 +901,7 @@ TEST(FunctionJsonbTEST, JsonbExtractIntTest) {
 
     check_function<DataTypeInt32, true>(func_name, input_types, data_set);
 
-    // jsonb_extract $[0].k1
+    // json_extract $[0].k1
     data_set = {
             {{Null(), STRING("$[0].k1")}, Null()},
             {{STRING("null"), STRING("$[0].k1")}, Null()},
@@ -918,10 +930,10 @@ TEST(FunctionJsonbTEST, JsonbExtractIntTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbExtractBigIntTest) {
-    std::string func_name = "jsonb_extract_bigint";
+    std::string func_name = "json_extract_bigint";
     InputTypeSet input_types = {TypeIndex::JSONB, TypeIndex::String};
 
-    // jsonb_extract root
+    // json_extract root
     DataSet data_set = {
             {{Null(), STRING("$")}, Null()},
             {{STRING("null"), STRING("$")}, Null()},
@@ -946,7 +958,7 @@ TEST(FunctionJsonbTEST, JsonbExtractBigIntTest) {
 
     check_function<DataTypeInt64, true>(func_name, input_types, data_set);
 
-    // jsonb_extract obejct
+    // json_extract obejct
     data_set = {
             {{Null(), STRING("$.k1")}, Null()},
             {{STRING("null"), STRING("$.k1")}, Null()},
@@ -971,7 +983,7 @@ TEST(FunctionJsonbTEST, JsonbExtractBigIntTest) {
 
     check_function<DataTypeInt64, true>(func_name, input_types, data_set);
 
-    // jsonb_extract array
+    // json_extract array
     data_set = {
             {{Null(), STRING("$[0]")}, Null()},
             {{STRING("null"), STRING("$[0]")}, Null()},
@@ -1032,7 +1044,7 @@ TEST(FunctionJsonbTEST, JsonbExtractBigIntTest) {
 
     check_function<DataTypeInt64, true>(func_name, input_types, data_set);
 
-    // jsonb_extract $[0].k1
+    // json_extract $[0].k1
     data_set = {
             {{Null(), STRING("$[0].k1")}, Null()},
             {{STRING("null"), STRING("$[0].k1")}, Null()},
@@ -1061,10 +1073,10 @@ TEST(FunctionJsonbTEST, JsonbExtractBigIntTest) {
 }
 
 TEST(FunctionJsonbTEST, JsonbExtractDoubleTest) {
-    std::string func_name = "jsonb_extract_double";
+    std::string func_name = "json_extract_double";
     InputTypeSet input_types = {TypeIndex::JSONB, TypeIndex::String};
 
-    // jsonb_extract root
+    // json_extract root
     DataSet data_set = {
             {{Null(), STRING("$")}, Null()},
             {{STRING("null"), STRING("$")}, Null()},
@@ -1089,7 +1101,7 @@ TEST(FunctionJsonbTEST, JsonbExtractDoubleTest) {
 
     check_function<DataTypeFloat64, true>(func_name, input_types, data_set);
 
-    // jsonb_extract obejct
+    // json_extract obejct
     data_set = {
             {{STRING("null"), STRING("$.k1")}, Null()},
             {{STRING("true"), STRING("$.k1")}, Null()},
@@ -1113,7 +1125,7 @@ TEST(FunctionJsonbTEST, JsonbExtractDoubleTest) {
 
     check_function<DataTypeFloat64, true>(func_name, input_types, data_set);
 
-    // jsonb_extract array
+    // json_extract array
     data_set = {
             {{STRING("null"), STRING("$[0]")}, Null()},
             {{STRING("true"), STRING("$[0]")}, Null()},
@@ -1173,7 +1185,7 @@ TEST(FunctionJsonbTEST, JsonbExtractDoubleTest) {
 
     check_function<DataTypeFloat64, true>(func_name, input_types, data_set);
 
-    // jsonb_extract $[0].k1
+    // json_extract $[0].k1
     data_set = {
             {{STRING("null"), STRING("$[0].k1")}, Null()},
             {{STRING("true"), STRING("$[0].k1")}, Null()},
@@ -1475,5 +1487,4 @@ TEST(FunctionJsonbTEST, JsonbCastFromOtherTest) {
             "CAST", {Notnull {TypeIndex::String}, ConstedNotnull {TypeIndex::JSONB}},
             {{{STRING(R"("abcd")"), Null()}, STRING(R"("abcd")")}});
 }
-
 } // namespace doris::vectorized
