@@ -107,11 +107,6 @@ public class CompoundPredicate extends Predicate {
         msg.setOpcode(op.toThrift());
     }
 
-    @Override
-    public boolean isVectorized() {
-        return false;
-    }
-
     @Override
     public void analyzeImpl(Analyzer analyzer) throws AnalysisException {
         super.analyzeImpl(analyzer);
@@ -278,13 +273,38 @@ public class CompoundPredicate extends Predicate {
         return hasNullableChild();
     }
 
-    @Override
-    public void finalizeImplForNereids() throws AnalysisException {
-
-    }
-
     @Override
     public String toString() {
         return toSqlImpl();
     }
+
+    @Override
+    public boolean containsSubPredicate(Expr subExpr) throws AnalysisException {
+        if (op.equals(Operator.AND)) {
+            for (Expr child : children) {
+                if (child.containsSubPredicate(subExpr)) {
+                    return true;
+                }
+            }
+        }
+        return super.containsSubPredicate(subExpr);
+    }
+
+    @Override
+    public Expr replaceSubPredicate(Expr subExpr) {
+        if (toSqlWithoutTbl().equals(subExpr.toSqlWithoutTbl())) {
+            return null;
+        }
+        if (op.equals(Operator.AND)) {
+            Expr lhs = children.get(0);
+            Expr rhs = children.get(1);
+            if (lhs.replaceSubPredicate(subExpr) == null) {
+                return rhs;
+            }
+            if (rhs.replaceSubPredicate(subExpr) == null) {
+                return lhs;
+            }
+        }
+        return this;
+    }
 }
