@@ -175,7 +175,7 @@ public class CreateRoutineLoadStmt extends DdlStmt {
 
     public static final Predicate<Long> DESIRED_CONCURRENT_NUMBER_PRED = (v) -> v > 0L;
     public static final Predicate<Long> MAX_ERROR_NUMBER_PRED = (v) -> v >= 0L;
-    public static final Predicate<Long> MAX_BATCH_INTERVAL_PRED = (v) -> v >= 5 && v <= 60;
+    public static final Predicate<Long> MAX_BATCH_INTERVAL_PRED = (v) -> v >= 1 && v <= 60;
     public static final Predicate<Long> MAX_BATCH_ROWS_PRED = (v) -> v >= 200000;
     public static final Predicate<Long> MAX_BATCH_SIZE_PRED = (v) -> v >= 100 * 1024 * 1024 && v <= 1024 * 1024 * 1024;
     public static final Predicate<Long> EXEC_MEM_LIMIT_PRED = (v) -> v >= 0L;
@@ -373,6 +373,9 @@ public class CreateRoutineLoadStmt extends DdlStmt {
                     // check where expr
                     ImportWhereStmt node = (ImportWhereStmt) parseNode;
                     if (node.isPreceding()) {
+                        if (isMultiTable) {
+                            throw new AnalysisException("Multi-table load does not support setting columns info");
+                        }
                         if (precedingImportWhereStmt != null) {
                             throw new AnalysisException("repeat setting of preceding where predicate");
                         }
@@ -429,7 +432,7 @@ public class CreateRoutineLoadStmt extends DdlStmt {
 
         maxBatchIntervalS = Util.getLongPropertyOrDefault(jobProperties.get(MAX_BATCH_INTERVAL_SEC_PROPERTY),
                 RoutineLoadJob.DEFAULT_MAX_INTERVAL_SECOND, MAX_BATCH_INTERVAL_PRED,
-                MAX_BATCH_INTERVAL_SEC_PROPERTY + " should between 5 and 60");
+                MAX_BATCH_INTERVAL_SEC_PROPERTY + " should between 1 and 60");
 
         maxBatchRows = Util.getLongPropertyOrDefault(jobProperties.get(MAX_BATCH_ROWS_PROPERTY),
                 RoutineLoadJob.DEFAULT_MAX_BATCH_ROWS, MAX_BATCH_ROWS_PRED,
