@@ -34,6 +34,7 @@ import org.apache.doris.clone.TabletScheduler.PathSlot;
 import org.apache.doris.common.Config;
 import org.apache.doris.common.FeConstants;
 import org.apache.doris.common.Pair;
+import org.apache.doris.common.util.DebugUtil;
 import org.apache.doris.common.util.TimeUtils;
 import org.apache.doris.persist.ReplicaPersistInfo;
 import org.apache.doris.resource.Tag;
@@ -127,7 +128,6 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         RUNNING, // tablet is being scheduled
         FINISHED, // task is finished
         CANCELLED, // task is failed
-        TIMEOUT, // task is timeout
         UNEXPECTED // other unexpected errors
     }
 
@@ -180,6 +180,8 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     private long visibleVersion = -1;
     private long committedVersion = -1;
 
+    private long tabletSize = 0;
+
     private Replica srcReplica = null;
     private long srcPathHash = -1;
     // for disk balance to keep src path, and avoid take slot on selectAlternativeTablets
@@ -282,6 +284,10 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         return failedSchedCounter;
     }
 
+    public void resetFailedSchedCounter() {
+        failedSchedCounter = 0;
+    }
+
     public void increaseFailedRunningCounter() {
         ++failedRunningCounter;
     }
@@ -302,7 +308,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         } else {
             decommissionTime = -1;
             if (code == SubCode.WAITING_SLOT && type != Type.BALANCE) {
-                return failedSchedCounter > 30 * 1000 / TabletScheduler.SCHEDULE_INTERVAL_MS;
+                return failedSchedCounter > 30 * 1000 / FeConstants.tablet_schedule_interval_ms;
             } else {
                 return failedSchedCounter > 10;
             }
@@ -478,13 +484,13 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
 
     // database lock should be held.
     public long getTabletSize() {
-        long max = Long.MIN_VALUE;
-        for (Replica replica : tablet.getReplicas()) {
-            if (replica.getDataSize() > max) {
-                max = replica.getDataSize();
-            }
-        }
-        return max;
+        return tabletSize;
+    }
+
+    public void updateTabletSize() {
+        tabletSize = 0;
+        tablet.getReplicas().stream().forEach(
+                replica -> tabletSize = Math.max(tabletSize, replica.getDataSize()));
     }
 
     /*
@@ -656,7 +662,9 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
      */
     public void chooseDestReplicaForVersionIncomplete(Map<Long, PathSlot> backendsWorkingSlots)
             throws SchedException {
-        List<Replica> candidates = Lists.newArrayList();
+        List<Replica> decommissionCand = Lists.newArrayList();
+        List<Replica> colocateCand = Lists.newArrayList();
+        List<Replica> notColocateCand = Lists.newArrayList();
         for (Replica replica : tablet.getReplicas()) {
             if (replica.isBad()) {
                 LOG.debug("replica {} is bad, skip. tablet: {}",
@@ -671,21 +679,35 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 continue;
             }
 
-            // check version and replica state.
-            // if the replica's state is DECOMMISSION, it may be chose as dest replica,
-            // and its state will be set to NORMAL later.
+            // not enough version completed replicas, then try add back the decommission replica.
+            if (replica.getState() == ReplicaState.DECOMMISSION) {
+                decommissionCand.add(replica);
+                continue;
+            }
+
             if (replica.getLastFailedVersion() <= 0
-                    && replica.getVersion() >= visibleVersion
-                    && replica.getState() != ReplicaState.DECOMMISSION) {
+                    && replica.getVersion() >= visibleVersion) {
                 // skip healthy replica
                 LOG.debug("replica {} version {} is healthy, visible version {}, replica state {}, skip. tablet: {}",
                         replica.getId(), replica.getVersion(), visibleVersion, replica.getState(), tabletId);
                 continue;
             }
 
-            candidates.add(replica);
+            if (colocateBackendsSet != null && colocateBackendsSet.contains(replica.getBackendId())) {
+                colocateCand.add(replica);
+            } else {
+                notColocateCand.add(replica);
+            }
         }
 
+        List<Replica> candidates = null;
+        if (!colocateCand.isEmpty()) {
+            candidates = colocateCand;
+        } else if (!notColocateCand.isEmpty()) {
+            candidates = notColocateCand;
+        } else {
+            candidates = decommissionCand;
+        }
         if (candidates.isEmpty()) {
             throw new SchedException(Status.UNRECOVERABLE, "unable to choose dest replica");
         }
@@ -748,7 +770,8 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             //
             // If we do not reset this replica state to NORMAL, the tablet's health status will be in VERSION_INCOMPLETE
             // forever, because the replica in the DECOMMISSION state will not receive the load task.
-            chosenReplica.setWatermarkTxnId(-1);
+            chosenReplica.setPreWatermarkTxnId(-1);
+            chosenReplica.setPostWatermarkTxnId(-1);
             chosenReplica.setState(ReplicaState.NORMAL);
             setDecommissionTime(-1);
             LOG.info("choose replica {} on backend {} of tablet {} as dest replica for version incomplete,"
@@ -889,6 +912,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         // if this is a balance task, or this is a repair task with
         // REPLICA_MISSING/REPLICA_RELOCATING,
         // we create a new replica with state CLONE
+        long replicaId = 0;
         if (tabletStatus == TabletStatus.REPLICA_MISSING
                 || tabletStatus == TabletStatus.REPLICA_RELOCATING || type == Type.BALANCE
                 || tabletStatus == TabletStatus.COLOCATE_MISMATCH
@@ -901,14 +925,9 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                     committedVersion, /* use committed version as last failed version */
                     -1 /* last success version */);
 
-            TBackend tSrcBe = new TBackend(srcBe.getHost(), srcBe.getBePort(), srcBe.getHttpPort());
-            cloneTask = new CloneTask(destBackendId, dbId, tblId, partitionId, indexId,
-                tabletId, cloneReplica.getId(), schemaHash, Lists.newArrayList(tSrcBe), storageMedium,
-                visibleVersion, (int) (taskTimeoutMs / 1000));
-            cloneTask.setPathHash(srcPathHash, destPathHash);
-
             // addReplica() method will add this replica to tablet inverted index too.
             tablet.addReplica(cloneReplica);
+            replicaId = cloneReplica.getId();
         } else if (tabletStatus == TabletStatus.VERSION_INCOMPLETE) {
             Preconditions.checkState(type == Type.REPAIR, type);
             // double check
@@ -921,18 +940,31 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 throw new SchedException(Status.SCHEDULE_FAILED, "dest replica's path hash is changed. "
                         + "current: " + replica.getPathHash() + ", scheduled: " + destPathHash);
             }
-
-            TBackend tSrcBe = new TBackend(srcBe.getHost(), srcBe.getBePort(), srcBe.getHttpPort());
-            cloneTask = new CloneTask(destBackendId, dbId, tblId, partitionId, indexId,
-                tabletId, replica.getId(), schemaHash, Lists.newArrayList(tSrcBe), storageMedium,
-                visibleVersion, (int) (taskTimeoutMs / 1000));
-            cloneTask.setPathHash(srcPathHash, destPathHash);
+            replicaId = replica.getId();
         }
 
+        TBackend tSrcBe = new TBackend(srcBe.getHost(), srcBe.getBePort(), srcBe.getHttpPort());
+        TBackend tDestBe = new TBackend(destBe.getHost(), destBe.getBePort(), destBe.getHttpPort());
+
+        cloneTask = new CloneTask(tDestBe, destBackendId, dbId, tblId, partitionId, indexId, tabletId,
+                replicaId, schemaHash, Lists.newArrayList(tSrcBe), storageMedium,
+                visibleVersion, (int) (taskTimeoutMs / 1000));
+        cloneTask.setPathHash(srcPathHash, destPathHash);
+
         this.state = State.RUNNING;
         return cloneTask;
     }
 
+    // for storage migration or cloning a new replica
+    public long getDestEstimatedCopingSize() {
+        if ((cloneTask != null && tabletStatus != TabletStatus.VERSION_INCOMPLETE)
+                || storageMediaMigrationTask != null) {
+            return Math.max(getTabletSize(), 10L);
+        } else {
+            return 0;
+        }
+    }
+
     // timeout is between MIN_CLONE_TASK_TIMEOUT_MS and MAX_CLONE_TASK_TIMEOUT_MS
     private long getApproximateTimeoutMs() {
         long tabletSize = getTabletSize();
@@ -1115,6 +1147,8 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         result.add(TimeUtils.longToTimeString(lastSchedTime));
         result.add(TimeUtils.longToTimeString(lastVisitedTime));
         result.add(TimeUtils.longToTimeString(finishedTime));
+        Pair<Double, String> tabletSizeDesc = DebugUtil.getByteUint(tabletSize);
+        result.add(DebugUtil.DECIMAL_FORMAT_SCALE_3.format(tabletSizeDesc.first) + " " + tabletSizeDesc.second);
         result.add(copyTimeMs > 0 ? String.valueOf(copySize / copyTimeMs / 1000.0) : FeConstants.null_string);
         result.add(String.valueOf(failedSchedCounter));
         result.add(String.valueOf(failedRunningCounter));
@@ -1142,8 +1176,13 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         value += (Priority.VERY_HIGH.ordinal() - priority.ordinal() + 1) * 60  * 1000L;
         value += 5000L * (failedSchedCounter / 10);
 
+        if (schedFailedCode == SubCode.WAITING_DECOMMISSION) {
+            value += 5 * 1000L;
+        }
+
+        // repair tasks always prior than balance
         if (type == Type.BALANCE) {
-            value += 30 * 60 * 1000L;
+            value += 10 * 24 * 3600L;
         }
 
         return value;
@@ -1154,12 +1193,19 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         StringBuilder sb = new StringBuilder();
         sb.append("tablet id: ").append(tabletId).append(", status: ").append(tabletStatus.name());
         sb.append(", state: ").append(state.name()).append(", type: ").append(type.name());
+        if (type == Type.BALANCE && balanceType != null) {
+            sb.append(", balance: ").append(balanceType.name());
+        }
+        if (priority != null) {
+            sb.append(", priority: ").append(priority.name());
+        }
+        sb.append(", tablet size: ").append(tabletSize);
         if (srcReplica != null) {
-            sb.append(". from backend: ").append(srcReplica.getBackendId());
+            sb.append(", from backend: ").append(srcReplica.getBackendId());
             sb.append(", src path hash: ").append(srcPathHash);
         }
         if (destPathHash != -1) {
-            sb.append(". to backend: ").append(destBackendId);
+            sb.append(", to backend: ").append(destBackendId);
             sb.append(", dest path hash: ").append(destPathHash);
         }
         sb.append(", visible version: ").append(visibleVersion);
@@ -1200,7 +1246,8 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 // any intermediate state it set during the scheduling process.
                 if (replica.getState() == ReplicaState.DECOMMISSION) {
                     replica.setState(ReplicaState.NORMAL);
-                    replica.setWatermarkTxnId(-1);
+                    replica.setPreWatermarkTxnId(-1);
+                    replica.setPostWatermarkTxnId(-1);
                     LOG.debug("reset replica {} on backend {} of tablet {} state from DECOMMISSION to NORMAL",
                             replica.getId(), replica.getBackendId(), tabletId);
                 }
