@@ -144,9 +144,8 @@ public class EsUtil {
             if (dorisMeta != null) {
                 JsonNode arrayNode = dorisMeta.get("array_fields");
                 if (arrayNode != null) {
-                    Iterator<JsonNode> iterator = arrayNode.iterator();
-                    while (iterator.hasNext()) {
-                        arrayFields.add(iterator.next().asText());
+                    for (JsonNode jsonNode : arrayNode) {
+                        arrayFields.add(jsonNode.asText());
                     }
                 }
             }
@@ -211,12 +210,31 @@ public class EsUtil {
         while (iterator.hasNext()) {
             String fieldName = iterator.next();
             ObjectNode fieldValue = (ObjectNode) mappingProps.get(fieldName);
-            Column column = parseEsField(fieldName, fieldValue, arrayFields);
+            Column column = parseEsField(fieldName, replaceFieldAlias(mappingProps, fieldValue), arrayFields);
             columns.add(column);
         }
         return columns;
     }
 
+    private static ObjectNode replaceFieldAlias(ObjectNode mappingProps, ObjectNode fieldValue) {
+        if (!fieldValue.has("type")) {
+            return fieldValue;
+        }
+        String typeStr = fieldValue.get("type").asText();
+        if ("alias".equals(typeStr)) {
+            String path = fieldValue.get("path").asText();
+            if ("_id".equals(path)) {
+                // _id is not in mappingProps, use keyword type.
+                fieldValue.put("type", "keyword");
+            } else {
+                if (mappingProps.has(path)) {
+                    return (ObjectNode) mappingProps.get(path);
+                }
+            }
+        }
+        return fieldValue;
+    }
+
     private static Column parseEsField(String fieldName, ObjectNode fieldValue, List<String> arrayFields) {
         Column column = new Column();
         column.setName(fieldName);
