@@ -106,6 +106,11 @@ public class BrokerFileGroup implements Writable {
     private boolean readJsonByLine = false;
     private boolean numAsString = false;
     private boolean trimDoubleQuotes = false;
+    private int skipLines;
+
+    private byte enclose;
+
+    private  byte escape;
 
     // for unit test and edit log persistence
     private BrokerFileGroup() {
@@ -210,6 +215,8 @@ public class BrokerFileGroup implements Writable {
         if (lineDelimiter == null) {
             lineDelimiter = "\n";
         }
+        enclose = dataDescription.getEnclose();
+        escape = dataDescription.getEscape();
 
         fileFormat = dataDescription.getFileFormat();
         if (fileFormat != null) {
@@ -264,6 +271,7 @@ public class BrokerFileGroup implements Writable {
             numAsString = dataDescription.isNumAsString();
         }
         trimDoubleQuotes = dataDescription.getTrimDoubleQuotes();
+        skipLines = dataDescription.getSkipLines();
     }
 
     public long getTableId() {
@@ -278,6 +286,14 @@ public class BrokerFileGroup implements Writable {
         return lineDelimiter;
     }
 
+    public byte getEnclose() {
+        return enclose;
+    }
+
+    public byte getEscape() {
+        return escape;
+    }
+
     public String getFileFormat() {
         return fileFormat;
     }
@@ -422,6 +438,10 @@ public class BrokerFileGroup implements Writable {
         return trimDoubleQuotes;
     }
 
+    public int getSkipLines() {
+        return skipLines;
+    }
+
     @Override
     public String toString() {
         StringBuilder sb = new StringBuilder();
