@@ -156,9 +156,7 @@ public class AggScalarSubQueryToWindowFunction extends DefaultPlanRewriter<JobCo
         return apply.isScalar()
                 && !apply.isMarkJoin()
                 && apply.right() instanceof LogicalAggregate
-                && apply.isCorrelated()
-                && apply.getSubCorrespondingConjunct().isPresent()
-                && apply.getSubCorrespondingConjunct().get() instanceof ComparisonPredicate;
+                && apply.isCorrelated();
     }
 
     /**
@@ -326,7 +324,17 @@ public class AggScalarSubQueryToWindowFunction extends DefaultPlanRewriter<JobCo
         // it's a simple case, but we may meet some complex cases in ut.
         // TODO: support compound predicate and multi apply node.
 
-        Expression windowFilterConjunct = apply.getSubCorrespondingConjunct().get();
+        Map<Boolean, Set<Expression>> conjuncts = filter.getConjuncts().stream()
+                .collect(Collectors.groupingBy(conjunct -> Sets
+                        .intersection(conjunct.getInputSlotExprIds(), agg.getOutputExprIdSet())
+                        .isEmpty(), Collectors.toSet()));
+        Set<Expression> correlatedConjuncts = conjuncts.get(false);
+        if (correlatedConjuncts.isEmpty() || correlatedConjuncts.size() > 1
+                || !(correlatedConjuncts.iterator().next() instanceof ComparisonPredicate)) {
+            //TODO: only support simple comparison predicate now
+            return filter;
+        }
+        Expression windowFilterConjunct = correlatedConjuncts.iterator().next();
         windowFilterConjunct = PlanUtils.maybeCommuteComparisonPredicate(
                 (ComparisonPredicate) windowFilterConjunct, apply.left());
 
@@ -349,13 +357,10 @@ public class AggScalarSubQueryToWindowFunction extends DefaultPlanRewriter<JobCo
         aggOutExpr = ExpressionUtils.replace(aggOutExpr, ImmutableMap
                 .of(functions.get(0), windowFunctionAlias.toSlot()));
 
-        // we change the child contains the original agg output to agg output expr.
-        // for comparison predicate, it is always the child(1), since we ensure the window agg slot is in child(0)
-        // for in predicate, we should extract the options and find the corresponding child.
-        windowFilterConjunct = windowFilterConjunct
-                .withChildren(windowFilterConjunct.child(0), aggOutExpr);
+        windowFilterConjunct = ExpressionUtils.replace(windowFilterConjunct,
+                ImmutableMap.of(aggOut.toSlot(), aggOutExpr));
 
-        LogicalFilter<Plan> newFilter = (LogicalFilter<Plan>) filter.withChildren(apply.left());
+        LogicalFilter<Plan> newFilter = filter.withConjunctsAndChild(conjuncts.get(true), apply.left());
         LogicalWindow<Plan> newWindow = new LogicalWindow<>(ImmutableList.of(windowFunctionAlias), newFilter);
         LogicalFilter<Plan> windowFilter = new LogicalFilter<>(ImmutableSet.of(windowFilterConjunct), newWindow);
         return windowFilter;
