@@ -104,9 +104,6 @@ public class InApplyToJoin extends OneRewriteRuleFactory {
                 predicate = new EqualTo(left, right);
             }
 
-            if (apply.getSubCorrespondingConjunct().isPresent()) {
-                predicate = ExpressionUtils.and(predicate, apply.getSubCorrespondingConjunct().get());
-            }
             List<Expression> conjuncts = ExpressionUtils.extractConjunction(predicate);
             if (((InSubquery) apply.getSubqueryExpr()).isNot()) {
                 return new LogicalJoin<>(
