@@ -55,6 +55,7 @@ public interface Plan extends TreeNode<Plan> {
     boolean canBind();
 
     default boolean bound() {
+        // TODO: avoid to use getLogicalProperties()
         return !(getLogicalProperties() instanceof UnboundLogicalProperties);
     }
 
@@ -62,12 +63,6 @@ public interface Plan extends TreeNode<Plan> {
         return getExpressions().stream().anyMatch(Expression::hasUnbound);
     }
 
-    default boolean childrenBound() {
-        return children()
-                .stream()
-                .allMatch(Plan::bound);
-    }
-
     default LogicalProperties computeLogicalProperties() {
         throw new IllegalStateException("Not support compute logical properties for " + getClass().getName());
     }
