@@ -315,11 +315,6 @@ public class SortNode extends PlanNode {
                 "[" + Joiner.on(" ").join(strings) + "]").addValue(super.debugString()).toString();
     }
 
-    @Override
-    public int getNumInstances() {
-        return children.get(0).getNumInstances();
-    }
-
     @Override
     public Set<SlotId> computeInputSlotIds(Analyzer analyzer) throws NotImplementedException {
         removeUnusedExprs();
