@@ -18,8 +18,11 @@
 package org.apache.doris.planner.external;
 
 public enum TableFormatType {
+    HIVE("hive"),
     ICEBERG("iceberg"),
-    HUDI("hudi");
+    HUDI("hudi"),
+    PAIMON("paimon"),
+    TRANSACTIONAL_HIVE("transactional_hive");
 
     private final String tableFormatType;
 
