@@ -54,7 +54,7 @@ import org.apache.doris.common.jmockit.Deencapsulation;
 import org.apache.doris.datasource.CatalogMgr;
 import org.apache.doris.datasource.InternalCatalog;
 import org.apache.doris.mysql.MysqlCommand;
-import org.apache.doris.mysql.privilege.PaloAuth;
+import org.apache.doris.mysql.privilege.AccessControllerManager;
 import org.apache.doris.system.SystemInfoService;
 import org.apache.doris.thrift.TStorageType;
 
@@ -88,7 +88,6 @@ public class ShowExecutorTest {
         ctx = new ConnectContext();
         ctx.setCommand(MysqlCommand.COM_SLEEP);
 
-
         Column column1 = new Column("col1", PrimitiveType.BIGINT);
         Column column2 = new Column("col2", PrimitiveType.DOUBLE);
         column1.setIsKey(true);
@@ -169,7 +168,7 @@ public class ShowExecutorTest {
         };
 
         // mock auth
-        PaloAuth auth = AccessTestUtil.fetchAdminAccess();
+        AccessControllerManager accessManager = AccessTestUtil.fetchAdminAccess();
 
         // mock catalog
         catalog = Deencapsulation.newInstance(InternalCatalog.class);
@@ -182,14 +181,6 @@ public class ShowExecutorTest {
                 catalog.getDbNullable("testCluster:emptyDb");
                 minTimes = 0;
                 result = null;
-
-                catalog.getClusterDbNames("testCluster");
-                minTimes = 0;
-                result = Lists.newArrayList("testCluster:testDb");
-
-                catalog.getClusterDbNames("");
-                minTimes = 0;
-                result = Lists.newArrayList("");
             }
         };
 
@@ -222,9 +213,9 @@ public class ShowExecutorTest {
                 minTimes = 0;
                 result = catalog;
 
-                env.getAuth();
+                env.getAccessManager();
                 minTimes = 0;
-                result = auth;
+                result = accessManager;
 
                 Env.getCurrentEnv();
                 minTimes = 0;
