@@ -90,7 +90,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testCommonFactors() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1=tb2.k1 and tb1.k2 =1) or (tb1.k1=tb2.k1 and tb2.k2=1)";
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1=tb2.k1 and tb1.k2 =1) or (tb1.k1=tb2.k1 and tb2.k2=1)";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("HASH JOIN"));
         Assert.assertEquals(1, StringUtils.countMatches(planString, "`tb1`.`k1` = `tb2`.`k1`"));
@@ -98,14 +98,14 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsWithOrPredicate() throws Exception {
-        String query = "select * from tb1 where tb1.k1 > 1000 or tb1.k1 < 200 or tb1.k1 = 300";
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1 where tb1.k1 > 1000 or tb1.k1 < 200 or tb1.k1 = 300";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("PREDICATES: `tb1`.`k1` = 300 OR `tb1`.`k1` > 1000 OR `tb1`.`k1` < 200"));
     }
 
     @Test
     public void testWideCommonFactorsWithEqualPredicate() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1=1 and tb2.k1=1) or (tb1.k1 =2 and tb2.k1=2)";
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1=1 and tb2.k1=1) or (tb1.k1 =2 and tb2.k1=2)";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` IN (1, 2)"));
         Assert.assertTrue(planString.contains("`tb2`.`k1` IN (1, 2)"));
@@ -122,7 +122,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsWithMergeRangePredicate() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1 between 1 and 3 and tb2.k1=1) or (tb1.k1 <2 and tb2.k2=2)";
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1 between 1 and 3 and tb2.k1=1) or (tb1.k1 <2 and tb2.k2=2)";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` <= 3"));
         Assert.assertTrue(planString.contains("NESTED LOOP JOIN"));
@@ -138,7 +138,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsWithDuplicateRangePredicate() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1 >1 and tb1.k1 >1 and tb1.k1 <5 and tb2.k1=1) "
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1 >1 and tb1.k1 >1 and tb1.k1 <5 and tb2.k1=1) "
                 + "or (tb1.k1 <2 and tb2.k2=2)";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` < 5"));
@@ -147,7 +147,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsWithInPredicate() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1 in (1) and tb2.k1 in(1)) "
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1 in (1) and tb2.k1 in(1)) "
                 + "or (tb1.k1 in(2) and tb2.k1 in(2))";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` IN (1, 2)"));
@@ -157,7 +157,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsWithDuplicateInPredicate() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1 in (1,2) and tb2.k1 in(1,2)) "
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1 in (1,2) and tb2.k1 in(1,2)) "
                 + "or (tb1.k1 in(3) and tb2.k1 in(2))";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` IN (1, 2, 3)"));
@@ -167,7 +167,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsWithRangeAndIn() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1 between 1 and 3 and tb2.k1 in(1,2)) "
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1 between 1 and 3 and tb2.k1 in(1,2)) "
                 + "or (tb1.k1 between 2 and 4 and tb2.k1 in(3))";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` >= 1"));
@@ -178,7 +178,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testWideCommonFactorsAndCommonFactors() throws Exception {
-        String query = "select * from tb1, tb2 where (tb1.k1 between 1 and 3 and tb1.k1=tb2.k1) "
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb1, tb2 where (tb1.k1 between 1 and 3 and tb1.k1=tb2.k1) "
                 + "or (tb1.k1=tb2.k1 and tb1.k1 between 2 and 4)";
         String planString = dorisAssert.query(query).explainQuery();
         Assert.assertTrue(planString.contains("`tb1`.`k1` >= 1"));
@@ -237,7 +237,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
                 + "\"storage_format\" = \"V2\"\n"
                 + ");";
         dorisAssert.withTable(createTableSQL);
-        String query = "select sum(l_extendedprice* (1 - l_discount)) as revenue "
+        String query = "select /*+ SET_VAR(enable_nereids_planner=false) */ sum(l_extendedprice* (1 - l_discount)) as revenue "
                 + "from lineitem, part "
                 + "where ( p_partkey = l_partkey and p_brand = 'Brand#11' "
                 + "and p_container in ('SM CASE', 'SM BOX', 'SM PACK', 'SM PKG') "
@@ -271,27 +271,27 @@ public class ExtractCommonFactorsRuleFunctionTest {
     @Test
     public void testRewriteLikePredicate() throws Exception {
         // tinyint
-        String sql = "select * from tb3 where k1 like '%4%';";
+        String sql = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb3 where k1 like '%4%';";
         LOG.info("EXPLAIN:{}", dorisAssert.query(sql).explainQuery());
         dorisAssert.query(sql).explainContains("`k1` LIKE '%4%'");
 
         // smallint
-        sql = "select * from tb3 where k2 like '%4%';";
+        sql = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb3 where k2 like '%4%';";
         LOG.info("EXPLAIN:{}", dorisAssert.query(sql).explainQuery());
         dorisAssert.query(sql).explainContains("`k2` LIKE '%4%'");
 
         // int
-        sql = "select * from tb3 where k3 like '%4%';";
+        sql = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb3 where k3 like '%4%';";
         LOG.info("EXPLAIN:{}", dorisAssert.query(sql).explainQuery());
         dorisAssert.query(sql).explainContains("`k3` LIKE '%4%'");
 
         // bigint
-        sql = "select * from tb3 where k4 like '%4%';";
+        sql = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb3 where k4 like '%4%';";
         LOG.info("EXPLAIN:{}", dorisAssert.query(sql).explainQuery());
         dorisAssert.query(sql).explainContains("`k4` LIKE '%4%'");
 
         // largeint
-        sql = "select * from tb3 where k5 like '%4%';";
+        sql = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from tb3 where k5 like '%4%';";
         LOG.info("EXPLAIN:{}", dorisAssert.query(sql).explainQuery());
         dorisAssert.query(sql).explainContains("`k5` LIKE '%4%'");
     }
@@ -327,7 +327,7 @@ public class ExtractCommonFactorsRuleFunctionTest {
 
     @Test
     public void testExtractCommonFactorsWithOnClause() throws Exception {
-        String sql = "select * from\n"
+        String sql = "select /*+ SET_VAR(enable_nereids_planner=false) */ * from\n"
                 + "db1.nation n1 join db1.nation n2\n"
                 + "on (n1.n_name = 'FRANCE' and n2.n_name = 'GERMANY')\n"
                 + "or (n1.n_name = 'GERMANY' and n2.n_name = 'FRANCE')";
